@@ -237,13 +237,13 @@ def Rop(f, wrt, eval_points, disconnected_outputs="raise", return_disconnected="
                 )
         except AttributeError:
             # wrt_elem and eval_point don't always have ndim like random type
-            # Tensor, Sparse and GpuArray have the ndim attribute
+            # Tensor, Sparse have the ndim attribute
             pass
 
     seen_nodes = OrderedDict()
 
     def _traverse(node):
-        """ TODO: writeme """
+        """TODO: writeme"""
 
         if node is None:
             return
@@ -301,7 +301,7 @@ def Rop(f, wrt, eval_points, disconnected_outputs="raise", return_disconnected="
                     # correctly, the same as grad
                     y = aesara.tensor.cast(y, x.type.dtype)
                     y = x.type.filter_variable(y)
-                assert x.type == y.type
+                assert x.type.in_same_class(y.type)
                 same_type_eval_points.append(y)
             else:
                 same_type_eval_points.append(y)
@@ -386,7 +386,7 @@ def Lop(f, wrt, eval_points, consider_constant=None, disconnected_inputs="raise"
         coordinates of the tensor element in the last
         If `f` is a list/tuple, then return a list/tuple with the results.
     """
-    if type(eval_points) not in (list, tuple):
+    if not isinstance(eval_points, (list, tuple)):
         eval_points = [eval_points]
 
     using_list = isinstance(wrt, list)
@@ -1050,7 +1050,7 @@ def _populate_grad_dict(var_to_app_to_idx, grad_dict, wrt, cost_name=None):
     term_dict = OrderedDict()
 
     def access_term_cache(node):
-        """ Populates term_dict[node] and returns it """
+        """Populates term_dict[node] and returns it"""
 
         if node not in term_dict:
 
@@ -1181,18 +1181,11 @@ def _populate_grad_dict(var_to_app_to_idx, grad_dict, wrt, cost_name=None):
                     ):
                         assert ng_dt == o_dt
 
-                # Someone who had obviously not read the Op contract tried
-                # to modify this part of the function.
-                # If you ever think it is a good idea to make an integer
-                # valued gradient, please
-                # 1) Read the Op contract again
-                # 2) Talk to Ian Goodfellow
-                # (Both of these sources will tell you not to do it)
-                for ng in new_output_grads:
-                    assert (
-                        getattr(ng.type, "dtype", None)
-                        not in aesara.tensor.type.discrete_dtypes
-                    )
+                assert all(
+                    getattr(ng.type, "dtype", None)
+                    not in aesara.tensor.type.discrete_dtypes
+                    for ng in new_output_grads
+                )
 
                 # If config.compute_test_value is turned on, check that the
                 # gradients on the outputs of this node have the right shape.
@@ -1324,7 +1317,7 @@ def _populate_grad_dict(var_to_app_to_idx, grad_dict, wrt, cost_name=None):
                         # The only other valid thing it can be is 0
 
                         is_zero = _is_zero(term)
-                        assert is_zero in ["yes", "no", "maybe"]
+                        assert is_zero in ("yes", "no", "maybe")
                         if is_zero == "maybe":
                             msg = (
                                 f"{node.op}.grad returned {term} of type {type(term)} for input"
@@ -1841,7 +1834,7 @@ def verify_grad(
     # random_projection should not have elements too small,
     # otherwise too much precision is lost in numerical gradient
     def random_projection():
-        plain = rng.rand(*o_fn_out.shape) + 0.5
+        plain = rng.random(o_fn_out.shape) + 0.5
         if cast_to_output_type and o_output.dtype == "float32":
             return np.array(plain, o_output.dtype)
         return plain
