@@ -1,24 +1,26 @@
 """Core graph classes."""
-import contextlib
 import warnings
 from collections import deque
 from copy import copy
 from itertools import count
 from typing import (
+    TYPE_CHECKING,
     Callable,
     Collection,
     Deque,
     Dict,
     Generator,
-    Hashable,
     Iterable,
+    Iterator,
     List,
     Optional,
+    Reversible,
     Sequence,
     Set,
     Tuple,
     TypeVar,
     Union,
+    cast,
 )
 
 import numpy as np
@@ -36,9 +38,13 @@ from aesara.graph.utils import (
 from aesara.misc.ordered_set import OrderedSet
 
 
-T = TypeVar("T")
+if TYPE_CHECKING:
+    from aesara.graph.type import Type
 
+
+T = TypeVar("T", bound="Node")
 NoParams = object()
+NodeAndChildren = Tuple[T, Optional[Iterable[T]]]
 
 
 class Node(MetaObject):
@@ -49,6 +55,8 @@ class Node(MetaObject):
     keeps track of its parents via `Variable.owner` / `Apply.inputs`.
 
     """
+    type: "Type"
+    name: Optional[str]
 
     def get_parents(self):
         """
@@ -63,20 +71,8 @@ class Node(MetaObject):
 class Apply(Node):
     """A `Node` representing the application of an operation to inputs.
 
-    An `Apply` instance serves as a simple structure with three important
-    attributes:
-
-    - :literal:`inputs` :  a list of `Variable` nodes that represent the
-      arguments of the expression,
-
-    - :literal:`outputs` : a list of `Variable` nodes that represent the
-      computed outputs of the expression, and
-
-    - :literal:`op` : an `Op` instance that determines the nature of the
-      expression being applied.
-
     Basically, an `Apply` instance is an object that represents the
-    Python statement `outputs = op(*inputs)`.
+    Python statement ``outputs = op(*inputs)``.
 
     This class is typically instantiated by a `Op.make_node` method, which
     is called by `Op.__call__`.
@@ -88,12 +84,6 @@ class Apply(Node):
     A `Linker` uses the `Apply` instance's `op` field to compute numeric values
     for the output variables.
 
-    Parameters
-    ----------
-    op : A Op instance
-    inputs : list of Variable instances
-    outputs : list of Variable instances
-
     Notes
     -----
     The `Variable.owner` field of each `Apply.outputs` element is set to `self`
@@ -102,11 +92,28 @@ class Apply(Node):
     If an output element has an owner that is neither `None` nor `self`, then a
     `ValueError` exception will be raised.
 
+    Attributes
+    ----------
+    op : Op
+        The operation that produces `outputs` given `inputs`.
+    inputs : List[Variable]
+        The arguments of the expression modeled by the `Apply` node.
+    outputs : List[Variable]
+        The outputs of the expression modeled by the `Apply` node.
+
     """
 
     def __init__(self, op, inputs, outputs):
+        """
+        Parameters
+        ----------
+        op : Op
+        inputs : List[Variable]
+        outputs : List[Variable]
+
+        """
         self.op = op
-        self.inputs = []
+        self.inputs: List[Variable] = []
         self.tag = Scratchpad()
 
         if not isinstance(inputs, (list, tuple)):
@@ -123,7 +130,7 @@ class Apply(Node):
                 raise TypeError(
                     f"The 'inputs' argument to Apply must contain Variable instances, not {input}"
                 )
-        self.outputs = []
+        self.outputs: List[Variable] = []
         # filter outputs to make sure each element is a Variable
         for i, output in enumerate(outputs):
             if isinstance(output, Variable):
@@ -187,21 +194,12 @@ class Apply(Node):
             raise ValueError(f"{self.op}.default_output is out of range.")
         return self.outputs[do]
 
-    out = property(default_output, doc="alias for self.default_output()")
-    """
-    Alias for self.default_output().
-
-    """
-
     def __str__(self):
         return op_as_string(self.inputs, self)
 
     def __repr__(self):
         return str(self)
 
-    def __asapply__(self):
-        return self
-
     def clone(self):
         """
         Duplicate this Apply instance with inputs = self.inputs.
@@ -231,8 +229,8 @@ class Apply(Node):
             List of `Variable` instances to use as inputs.
         strict : bool
             If ``True``, the type fields of all the inputs must be equal
-            to the current ones (or compatible, for instance `Tensor` /
-            `GpuArray` of the same dtype and broadcastable patterns,
+            to the current ones (or compatible, for instance `TensorType`
+            of the same dtype and broadcastable patterns,
             in which case they will be converted into current `Type`), and
             returned outputs are guaranteed to have the same types as
             ``self.outputs``.  If ``False``, then there's no guarantee that the
@@ -266,20 +264,24 @@ class Apply(Node):
     def get_parents(self):
         return list(self.inputs)
 
-    # convenience properties
-    nin = property(lambda self: len(self.inputs), doc="same as len(self.inputs)")
-    """
-    Property: Number of inputs.
+    @property
+    def out(self):
+        """An alias for `self.default_output`"""
+        return self.default_output()
 
-    """
-    nout = property(lambda self: len(self.outputs), doc="same as len(self.outputs)")
-    """
-    Property: Number of outputs.
+    @property
+    def nin(self):
+        """The number of inputs."""
+        return len(self.inputs)
 
-    """
-    params_type = property(
-        lambda self: self.op.params_type, doc="type to use for the params"
-    )
+    @property
+    def nout(self):
+        """The number of outputs."""
+        return len(self.outputs)
+
+    @property
+    def params_type(self):
+        return self.op.params_type
 
 
 class Variable(Node):
@@ -326,9 +328,6 @@ class Variable(Node):
     - `SparseVariable`: a subclass of `Variable` that represents
       a ``scipy.sparse.{csc,csr}_matrix`` object.
 
-    - `GpuArrayVariable`: a subclass of `Variable` that represents our object on
-      the GPU that is a subset of ``numpy.ndarray``.
-
     - `RandomVariable`.
 
     A `Variable` which is the output of a symbolic computation will have an owner
@@ -383,7 +382,23 @@ class Variable(Node):
     # __slots__ = ['type', 'owner', 'index', 'name']
     __count__ = count(0)
 
-    def __init__(self, type, owner=None, index=None, name=None):
+    _owner: Optional[Apply]
+
+    @property
+    def owner(self) -> Optional[Apply]:
+        return self._owner
+
+    @owner.setter
+    def owner(self, value) -> None:
+        self._owner = value
+
+    def __init__(
+        self,
+        type,
+        owner: Optional[Apply] = None,
+        index: Optional[int] = None,
+        name: Optional[str] = None,
+    ):
         super().__init__()
 
         self.tag = ValidatingScratchpad("test_value", type.filter)
@@ -392,7 +407,7 @@ class Variable(Node):
 
         if owner is not None and not isinstance(owner, Apply):
             raise TypeError("owner must be an Apply instance", owner)
-        self.owner = owner
+        self._owner = owner
 
         if index is not None and not isinstance(index, int):
             raise TypeError("index must be an int", index)
@@ -404,8 +419,6 @@ class Variable(Node):
 
         self.auto_name = "auto_" + str(next(self.__count__))
 
-        Variable.notify_construction_observers(self)
-
     def get_test_value(self):
         """Get the test value.
 
@@ -568,22 +581,6 @@ class Variable(Node):
             d["tag"] = t
         return d
 
-    #  refer to doc in nodes_constructed.
-    construction_observers: List = []
-
-    @classmethod
-    def append_construction_observer(cls, observer):
-        cls.construction_observers.append(observer)
-
-    @classmethod
-    def remove_construction_observer(cls, observer):
-        cls.construction_observers.remove(observer)
-
-    @classmethod
-    def notify_construction_observers(cls, instance):
-        for observer in cls.construction_observers:
-            observer(instance)
-
 
 class Constant(Variable):
     """A `Variable` with a fixed `data` field.
@@ -628,29 +625,18 @@ class Constant(Variable):
             return f"{type(self).__name__}{{{name}}}"
 
     def clone(self):
-        """Create a shallow clone.
+        """Return `self`, because there's no reason to clone a constant."""
+        return self
 
-        We clone this object, but we don't clone the data to lower memory
-        requirement. We suppose that the data will never change.
+    @property
+    def owner(self) -> None:
+        return None
 
-        """
-        cp = self.__class__(self.type, self.data, self.name)
-        cp.tag = copy(self.tag)
-        return cp
-
-    def __set_owner(self, value):
-        """Prevent the :prop:`owner` property from being set.
-
-        Raises
-        ------
-        ValueError
-            If `value` is not ``None``.
-
-        """
+    @owner.setter
+    def owner(self, value) -> None:
         if value is not None:
             raise ValueError("Constant instances cannot have an owner.")
 
-    owner = property(lambda self: None, __set_owner)
     value = property(lambda self: self.data, doc="read-only data access method")
 
     # index is not defined, because the `owner` attribute must necessarily be None
@@ -658,34 +644,30 @@ class Constant(Variable):
 
 def walk(
     nodes: Iterable[T],
-    expand: Callable[[T], Optional[Sequence[T]]],
+    expand: Callable[[T], Optional[Iterable[T]]],
     bfs: bool = True,
     return_children: bool = False,
-    hash_fn: Callable[[T], Hashable] = id,
-) -> Generator[T, None, Dict[T, List[T]]]:
-    """Walk through a graph, either breadth- or depth-first.
+    hash_fn: Callable[[T], int] = id,
+) -> Generator[Union[T, NodeAndChildren], None, None]:
+    r"""Walk through a graph, either breadth- or depth-first.
 
     Parameters
     ----------
-    nodes : deque
+    nodes
         The nodes from which to start walking.
-    expand : callable
+    expand
         A callable that is applied to each node in `nodes`, the results of
         which are either new nodes to visit or ``None``.
-    bfs : bool
+    bfs
         If ``True``, breath first search is used; otherwise, depth first
         search.
-    return_children : bool
+    return_children
         If ``True``, each output node will be accompanied by the output of
         `expand` (i.e. the corresponding child nodes).
-    hash_fn : callable
+    hash_fn
         The function used to produce hashes of the elements in `nodes`.
         The default is ``id``.
 
-    Yields
-    ------
-    nodes
-
     Notes
     -----
     A node will appear at most once in the return value, even if it
@@ -695,7 +677,7 @@ def walk(
 
     nodes = deque(nodes)
 
-    rval_set: Set[T] = set()
+    rval_set: Set[int] = set()
 
     nodes_pop: Callable[[], T]
     if bfs:
@@ -706,13 +688,13 @@ def walk(
     while nodes:
         node: T = nodes_pop()
 
-        node_hash: Hashable = hash_fn(node)
+        node_hash: int = hash_fn(node)
 
         if node_hash not in rval_set:
 
             rval_set.add(node_hash)
 
-            new_nodes: Optional[Sequence[T]] = expand(node)
+            new_nodes: Optional[Iterable[T]] = expand(node)
 
             if return_children:
                 yield node, new_nodes
@@ -724,7 +706,7 @@ def walk(
 
 
 def ancestors(
-    graphs: Iterable[Variable], blockers: Collection[Variable] = None
+    graphs: Iterable[Variable], blockers: Optional[Collection[Variable]] = None
 ) -> Generator[Variable, None, None]:
     r"""Return the variables that contribute to those in given graphs (inclusive).
 
@@ -745,15 +727,15 @@ def ancestors(
 
     """
 
-    def expand(r):
+    def expand(r: Variable) -> Optional[Iterator[Variable]]:
         if r.owner and (not blockers or r not in blockers):
             return reversed(r.owner.inputs)
 
-    yield from walk(graphs, expand, False)
+    yield from cast(Generator[Variable, None, None], walk(graphs, expand, False))
 
 
 def graph_inputs(
-    graphs: Iterable[Variable], blockers: Collection[Variable] = None
+    graphs: Iterable[Variable], blockers: Optional[Collection[Variable]] = None
 ) -> Generator[Variable, None, None]:
     r"""Return the inputs required to compute the given Variables.
 
@@ -782,26 +764,25 @@ def vars_between(
 
     Parameters
     ----------
-    ins : list
+    ins
         Input `Variable`\s.
-    outs : list
+    outs
         Output `Variable`\s.
 
     Yields
     ------
-    `Variable`\s
-        The `Variable`\s that are involved in the subgraph that lies
-        between `ins` and `outs`. This includes `ins`, `outs`,
-        ``orphans_between(ins, outs)`` and all values of all intermediary steps from
-        `ins` to `outs`.
+    The `Variable`\s that are involved in the subgraph that lies
+    between `ins` and `outs`. This includes `ins`, `outs`,
+    ``orphans_between(ins, outs)`` and all values of all intermediary steps from
+    `ins` to `outs`.
 
     """
 
-    def expand(r):
+    def expand(r: Variable) -> Optional[Iterable[Variable]]:
         if r.owner and r not in ins:
             return reversed(r.owner.inputs + r.owner.outputs)
 
-    yield from walk(outs, expand)
+    yield from cast(Generator[Variable, None, None], walk(outs, expand))
 
 
 def orphans_between(
@@ -893,16 +874,18 @@ def clone(
     if copy_orphans is None:
         copy_orphans = copy_inputs
     equiv = clone_get_equiv(inputs, outputs, copy_inputs, copy_orphans)
-    return [equiv[input] for input in inputs], [equiv[output] for output in outputs]
+    return [cast(Variable, equiv[input]) for input in inputs], [
+        cast(Variable, equiv[output]) for output in outputs
+    ]
 
 
 def clone_get_equiv(
-    inputs: List[Variable],
-    outputs: List[Variable],
+    inputs: Sequence[Variable],
+    outputs: Sequence[Variable],
     copy_inputs: bool = True,
     copy_orphans: bool = True,
-    memo: Optional[Dict[Variable, Variable]] = None,
-):
+    memo: Optional[Dict[Node, Node]] = None,
+) -> Dict[Node, Node]:
     """
     Return a dictionary that maps from `Variable` and `Apply` nodes in the
     original graph to a new node (a clone) in a new graph.
@@ -965,11 +948,13 @@ def clone_get_equiv(
 
 
 def clone_replace(
-    output: Collection[Variable],
-    replace: Optional[Dict[Variable, Variable]] = None,
+    output: List[Variable],
+    replace: Optional[
+        Union[Iterable[Tuple[Variable, Variable]], Dict[Variable, Variable]]
+    ] = None,
     strict: bool = True,
     share_inputs: bool = True,
-) -> Collection[Variable]:
+) -> List[Variable]:
     """Clone a graph and replace subgraphs within it.
 
     It returns a copy of the initial subgraph with the corresponding
@@ -990,6 +975,7 @@ def clone_replace(
     """
     from aesara.compile.function.pfunc import rebuild_collect_shared
 
+    items: Union[List[Tuple[Variable, Variable]], Tuple[Tuple[Variable, Variable], ...]]
     if isinstance(replace, dict):
         items = list(replace.items())
     elif isinstance(replace, (list, tuple)):
@@ -1015,13 +1001,15 @@ def clone_replace(
         _outs, [], new_replace, [], strict, share_inputs
     )
 
-    return outs
+    return cast(List[Variable], outs)
 
 
 def general_toposort(
     outputs: Iterable[T],
     deps: Callable[[T], Union[OrderedSet, List[T]]],
-    compute_deps_cache: Optional[Callable[[T], Union[OrderedSet, List[T]]]] = None,
+    compute_deps_cache: Optional[
+        Callable[[T], Optional[Union[OrderedSet, List[T]]]]
+    ] = None,
     deps_cache: Optional[Dict[T, List[T]]] = None,
     clients: Optional[Dict[T, List[T]]] = None,
 ) -> List[T]:
@@ -1061,7 +1049,7 @@ def general_toposort(
         if deps_cache is None:
             deps_cache = {}
 
-        def compute_deps_cache(io):
+        def _compute_deps_cache(io):
             if io not in deps_cache:
                 d = deps(io)
 
@@ -1079,23 +1067,27 @@ def general_toposort(
             else:
                 return deps_cache[io]
 
+    else:
+        _compute_deps_cache = compute_deps_cache
+
     if deps_cache is None:
         raise ValueError("deps_cache cannot be None")
 
-    search_res: List[T, Optional[List[T]]] = list(
-        walk(outputs, compute_deps_cache, bfs=False, return_children=True)
+    search_res: List[NodeAndChildren] = cast(
+        List[NodeAndChildren],
+        list(walk(outputs, _compute_deps_cache, bfs=False, return_children=True)),
     )
 
     _clients: Dict[T, List[T]] = {}
     sources: Deque[T] = deque()
     search_res_len: int = 0
-    for node, children in search_res:
+    for snode, children in search_res:
         search_res_len += 1
         if children:
             for child in children:
-                _clients.setdefault(child, []).append(node)
-        if not deps_cache.get(node):
-            sources.append(node)
+                _clients.setdefault(child, []).append(snode)
+        if not deps_cache.get(snode):
+            sources.append(snode)
 
     if clients is not None:
         clients.update(_clients)
@@ -1120,8 +1112,8 @@ def general_toposort(
 
 
 def io_toposort(
-    inputs: List[Variable],
-    outputs: List[Variable],
+    inputs: Iterable[Variable],
+    outputs: Reversible[Variable],
     orderings: Optional[Dict[Apply, List[Apply]]] = None,
     clients: Optional[Dict[Variable, List[Variable]]] = None,
 ) -> List[Apply]:
@@ -1332,8 +1324,8 @@ def as_string(
 
     orph = list(orphans_between(i, outputs))
 
-    multi: Set = set()
-    seen: Set = set()
+    multi = set()
+    seen = set()
     for output in outputs:
         op = output.owner
         if op in seen:
@@ -1349,11 +1341,11 @@ def as_string(
                 multi.add(op2)
             else:
                 seen.add(input.owner)
-    multi: Set = [x for x in multi]
+    multi_list = [x for x in multi]
     done: Set = set()
 
     def multi_index(x):
-        return multi.index(x) + 1
+        return multi_list.index(x) + 1
 
     def describe(r):
         if r.owner is not None and r not in i and r not in orph:
@@ -1368,7 +1360,7 @@ def as_string(
             else:
                 done.add(op)
                 s = node_formatter(op, [describe(input) for input in op.inputs])
-                if op in multi:
+                if op in multi_list:
                     return f"*{multi_index(op)} -> {s}"
                 else:
                     return s
@@ -1411,14 +1403,21 @@ def list_of_nodes(
         Output `Variable`\s.
 
     """
+
+    def expand(o: Apply) -> List[Apply]:
+        return [
+            inp.owner
+            for inp in o.inputs
+            if inp.owner and not any(i in inp.owner.outputs for i in inputs)
+        ]
+
     return list(
-        walk(
-            [o.owner for o in outputs],
-            lambda o: [
-                inp.owner
-                for inp in o.inputs
-                if inp.owner and not any(i in inp.owner.outputs for i in inputs)
-            ],
+        cast(
+            Iterable[Apply],
+            walk(
+                [o.owner for o in outputs if o.owner],
+                expand,
+            ),
         )
     )
 
@@ -1454,43 +1453,12 @@ def is_in_ancestors(l_apply: Apply, f_node: Apply) -> bool:
     return False
 
 
-@contextlib.contextmanager
-def nodes_constructed():
-    r"""
-    A context manager that is used in ``inherit_stack_trace`` and keeps track
-    of all the newly created variable nodes inside an optimization. A list
-    of ``new_nodes`` is instantiated but will be filled in a lazy manner (when
-    ``Variable.notify_construction_observers`` is called).
-
-
-    ``observer`` is the entity that updates the ``new_nodes`` list.
-    ``construction_observers`` is a list inside `Variable` class and contains
-    a list of observer functions. The observer functions inside
-    ``construction_observers`` are only called when a `Variable` is
-    instantiated (where ``Variable.notify_construction_observers`` is called).
-    When the observer function is called, a new `Variable` is added to
-    the `new_nodes` list.
-
-
-    Parameters
-    ----------
-    new_nodes
-        A list of all the `Variable`\s that are created inside the optimization.
-
-    yields
-        ``new_nodes`` list.
-    """
-    new_nodes = []
-
-    def observer(node):
-        new_nodes.append(node)
-
-    Variable.append_construction_observer(observer)
-    yield new_nodes
-    Variable.remove_construction_observer(observer)
-
-
-def equal_computations(xs, ys, in_xs=None, in_ys=None):
+def equal_computations(
+    xs: List[Union[np.ndarray, Variable]],
+    ys: List[Union[np.ndarray, Variable]],
+    in_xs: Optional[List[Variable]] = None,
+    in_ys: Optional[List[Variable]] = None,
+) -> bool:
     """Checks if Aesara graphs represent the same computations.
 
     The two lists `xs`, `ys` should have the same number of entries. The
@@ -1525,22 +1493,20 @@ def equal_computations(xs, ys, in_xs=None, in_ys=None):
 
     for x, y in zip(xs, ys):
         if not isinstance(x, Variable) and not isinstance(y, Variable):
-            return np.array_equal(x, y)
+            return cast(bool, np.array_equal(x, y))
         if not isinstance(x, Variable):
             if isinstance(y, Constant):
-                return np.array_equal(y.data, x)
+                return cast(bool, np.array_equal(y.data, x))
             return False
         if not isinstance(y, Variable):
             if isinstance(x, Constant):
-                return np.array_equal(x.data, y)
-            return False
-        if not isinstance(y, Variable):
+                return cast(bool, np.array_equal(x.data, y))
             return False
         if x.owner and not y.owner:
             return False
         if y.owner and not x.owner:
             return False
-        if x.owner:  # Check above tell that y.owner eval to True too.
+        if x.owner and y.owner:
             if x.owner.outputs.index(x) != y.owner.outputs.index(y):
                 return False
         if x not in in_xs and not (y.type.in_same_class(x.type)):
@@ -1554,8 +1520,9 @@ def equal_computations(xs, ys, in_xs=None, in_ys=None):
             return False
 
     common = set(zip(in_xs, in_ys))
-    different = set()
+    different: Set[Tuple[Variable, Variable]] = set()
     for dx, dy in zip(xs, ys):
+        assert isinstance(dx, Variable)
         # We checked above that both dx and dy have an owner or not
         if not dx.owner:
             if isinstance(dx, Constant) and isinstance(dy, Constant):
@@ -1579,6 +1546,8 @@ def equal_computations(xs, ys, in_xs=None, in_ys=None):
         results to be more efficient.
 
         """
+        if nd_x is nd_y:
+            return True
 
         if nd_x.op != nd_y.op:
             return False
@@ -1640,10 +1609,12 @@ def equal_computations(xs, ys, in_xs=None, in_ys=None):
 
     # Validate that each xs[i], ys[i] pair represents the same computation
     for i in range(len(xs)):
-        if xs[i].owner:
+        x_i: Variable = cast(Variable, xs[i])
+        if x_i.owner:
+            y_i: Variable = cast(Variable, ys[i])
             # The case where pairs of x[i]s and y[i]s don't both have an owner
             # have already been addressed.
-            is_equal = compare_nodes(xs[i].owner, ys[i].owner, common, different)
+            is_equal = compare_nodes(x_i.owner, y_i.owner, common, different)
             if not is_equal:
                 return False
 
@@ -1652,7 +1623,7 @@ def equal_computations(xs, ys, in_xs=None, in_ys=None):
 
 def get_var_by_name(
     graphs: Iterable[Variable], target_var_id: str, ids: str = "CHAR"
-) -> Tuple[Variable]:
+) -> Tuple[Variable, ...]:
     r"""Get variables in a graph using their names.
 
     Parameters
@@ -1670,17 +1641,18 @@ def get_var_by_name(
     """
     from aesara.graph.op import HasInnerGraph
 
-    def expand(r):
+    def expand(r) -> Optional[List[Variable]]:
         if r.owner:
-            res = r.owner.inputs
+            res = list(r.owner.inputs)
 
             if isinstance(r.owner.op, HasInnerGraph):
                 res.extend(r.owner.op.inner_outputs)
 
             return res
 
-    results = ()
+    results: Tuple[Variable, ...] = ()
     for var in walk(graphs, expand, False):
+        var = cast(Variable, var)
         if target_var_id == var.name or target_var_id == var.auto_name:
             results += (var,)
 
