@@ -13,7 +13,7 @@ from contextlib import contextmanager
 from copy import copy
 from functools import reduce
 from io import IOBase, StringIO
-from typing import Callable, Dict, Iterable, List, Optional, Union
+from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
 
 import numpy as np
 
@@ -168,9 +168,9 @@ def debugprint(
         used_ids = dict()
 
     results_to_print = []
-    profile_list = []
-    order = []  # Toposort
-    smap = []  # storage_map
+    profile_list: List[Optional[Any]] = []
+    order: List[Optional[List[Apply]]] = []  # Toposort
+    smap: List[Optional[StorageMapType]] = []  # storage_map
 
     if isinstance(obj, (list, tuple, set)):
         lobj = obj
@@ -329,6 +329,7 @@ N.B.:
         return _file.getvalue()
     else:
         _file.flush()
+    return _file
 
 
 def _debugprint(
@@ -396,7 +397,7 @@ def _debugprint(
         Internal. Used to pass information when recursing.
     """
     if depth == 0:
-        return
+        return file
 
     if order is None:
         order = []
@@ -881,8 +882,8 @@ default_printer = DefaultPrinter()
 
 class PPrinter(Printer):
     def __init__(self):
-        self.printers = []
-        self.printers_dict = {}
+        self.printers: List[Tuple[Union[Op, type, Callable], Printer]] = []
+        self.printers_dict: Dict[Union[Op, type, Callable], Printer] = {}
 
     def assign(self, condition: Union[Op, type, Callable], printer: Printer):
         if isinstance(condition, (Op, type)):
@@ -903,6 +904,7 @@ class PPrinter(Printer):
         for condition, printer in self.printers:
             if condition(pstate, r):
                 return printer.process(r, pstate)
+        return ""
 
     def clone(self):
         cp = copy(self)
@@ -999,7 +1001,7 @@ else:
     )
 
 
-pprint = PPrinter()
+pprint: PPrinter = PPrinter()
 pprint.assign(lambda pstate, r: True, default_printer)
 pprint.assign(lambda pstate, r: isinstance(r, Constant), constant_printer)
 
@@ -1012,8 +1014,6 @@ Print to the terminal a math-like expression.
 # colors not used: orange, amber#FFBF00, purple, pink,
 # used by default: green, blue, grey, red
 default_colorCodes = {
-    "GpuFromHost": "red",
-    "HostFromGpu": "red",
     "Scan": "yellow",
     "Shape": "brown",
     "IfElse": "magenta",
@@ -1705,7 +1705,7 @@ def get_node_by_id(
     if isinstance(graphs, Variable):
         graphs = (graphs,)
 
-    used_ids = dict()
+    used_ids: Dict[Variable, str] = {}
 
     _ = debugprint(graphs, file="str", used_ids=used_ids, ids=ids)
 
