@@ -20,14 +20,14 @@ import numpy as np
 from aesara import function, gradient
 from aesara import scalar as aes
 from aesara import shared
-from aesara import tensor as aet
+from aesara import tensor as at
 from aesara.compile import optdb
 from aesara.configdefaults import config
 from aesara.gradient import undefined_grad
 from aesara.graph.basic import Apply, Constant, Variable
-from aesara.graph.op import COp, Op
 from aesara.graph.opt import in2out, local_optimizer
-from aesara.graph.params_type import ParamsType
+from aesara.link.c.op import COp, Op
+from aesara.link.c.params_type import ParamsType
 from aesara.sandbox import multinomial
 from aesara.scalar import bool as bool_t
 from aesara.scalar import int32 as int_t
@@ -268,13 +268,13 @@ def ff_2p72(rstate):
 def mrg_next_value(rstate, new_rstate, NORM, mask, offset):
     # TODO : need description for method, parameter and return
     x11, x12, x13, x21, x22, x23 = rstate
-    assert type(x11) == np.int32
+    assert isinstance(x11, np.int32)
 
     i0, i7, i9, i15, i16, i22, i24 = np_int32_vals
     # first component
     y1 = ((x12 & MASK12) << i22) + (x12 >> i9) + ((x13 & MASK13) << i7) + (x13 >> i24)
 
-    assert type(y1) == np.int32
+    assert isinstance(y1, np.int32)
     if y1 < 0 or y1 >= M1:  # must also check overflow
         y1 -= M1
     y1 += x13
@@ -287,11 +287,11 @@ def mrg_next_value(rstate, new_rstate, NORM, mask, offset):
 
     # second component
     y1 = ((x21 & MASK2) << i15) + (MULT2 * (x21 >> i16))
-    assert type(y1) == np.int32
+    assert isinstance(y1, np.int32)
     if y1 < 0 or y1 >= M2:
         y1 -= M2
     y2 = ((x23 & MASK2) << i15) + (MULT2 * (x23 >> i16))
-    assert type(y2) == np.int32
+    assert isinstance(y2, np.int32)
     if y2 < 0 or y2 >= M2:
         y2 -= M2
     y2 += x23
@@ -373,8 +373,8 @@ class mrg_uniform(COp, mrg_uniform_base):
         # call through MRG_RandomStream instead.
         broad = []
         for i in range(self.output_type.ndim):
-            broad.append(aet.extract_constant(size[i]) == 1)
-        output_type = self.output_type.clone(broadcastable=broad)()
+            broad.append(at.extract_constant(size[i]) == 1)
+        output_type = self.output_type.clone(shape=broad)()
         rstate = as_tensor_variable(rstate)
         size = as_tensor_variable(size)
         return Apply(self, [rstate, size], [rstate.type(), output_type])
@@ -394,13 +394,13 @@ class mrg_uniform(COp, mrg_uniform_base):
         for s in size:
             n_elements *= s
         if n_elements > M1:
-            # The limit is on the C and GPU code. This perform don't
+            # The limit is on the C code. This perform don't
             # have this limit.  But to have all of them behave the
             # same (and have DebugMode don't use too much memory for
             # some rng_mrg tests) I also add this limit here.
             raise ValueError("rng_mrg does not support more then (2**31 -1) samples")
 
-        rstate = np.asarray(rstate)  # bring state from GPU if necessary
+        rstate = np.asarray(rstate)  # bring state from XXX if necessary
         if not self.inplace:
             rstate = rstate.copy()
 
@@ -527,8 +527,7 @@ class mrg_uniform(COp, mrg_uniform_base):
 
     def c_code(self, node, name, inp, out, sub):
         # If we try to use the C code here with something else than a
-        # TensorType, something is wrong (likely one of the GPU ops
-        # not defining C code correctly).
+        # TensorType, something is wrong.
         assert isinstance(node.inputs[0].type, TensorType)
         if self.output_type.dtype == "float16":
             # C code is not tested, fall back to Python
@@ -672,7 +671,7 @@ def guess_n_streams(size, warn=False):
     # TODO: a smart way of choosing the number of streams, see #612.
     # Note that this code was moved out of `MRG_RandomStream` so that it can
     # be easily accessed from tests, where we want to disable the warning.
-    if isinstance(size, (tuple, list)) and all([isinstance(i, int) for i in size]):
+    if isinstance(size, (tuple, list)) and all(isinstance(i, int) for i in size):
         # We can make a guess.
         r = 1
         for s in size:
@@ -736,7 +735,7 @@ class MRG_RandomStream:
 
     def set_rstate(self, seed):
         # TODO : need description for method, parameter
-        if isinstance(seed, int):
+        if isinstance(seed, (int, np.int32, np.int64)):
             if seed == 0:
                 raise ValueError("seed should not be 0", seed)
             elif seed >= M2:
@@ -808,7 +807,7 @@ class MRG_RandomStream:
 
         """
         assert isinstance(dtype, str)
-        assert n_streams < 2 ** 72
+        assert n_streams < 2**72
         assert n_streams > 0
         rval = np.zeros((n_streams, 6), dtype="int32")
         rval[0] = self.rstate
@@ -893,8 +892,8 @@ class MRG_RandomStream:
 
         if isinstance(size, tuple):
             msg = "size must be a tuple of int or an Aesara variable"
-            assert all([isinstance(i, (np.integer, int, Variable)) for i in size]), msg
-            if any([isinstance(i, (np.integer, int)) and i <= 0 for i in size]):
+            assert all(isinstance(i, (np.integer, int, Variable)) for i in size), msg
+            if any(isinstance(i, (np.integer, int)) and i <= 0 for i in size):
                 raise ValueError(
                     "The specified size contains a dimension with value <= 0", size
                 )
@@ -987,7 +986,7 @@ class MRG_RandomStream:
         pvals = as_tensor_variable(pvals)
         pvals = undefined_grad(pvals)
         if size is not None:
-            if any([isinstance(i, int) and i <= 0 for i in size]):
+            if any(isinstance(i, int) and i <= 0 for i in size):
                 raise ValueError(
                     "The specified size contains a dimension with value <= 0", size
                 )
@@ -1172,8 +1171,8 @@ class MRG_RandomStream:
         if dtype is None:
             dtype = aes.upcast(config.floatX, avg.dtype, std.dtype)
 
-        avg = aet.cast(avg, dtype=dtype)
-        std = aet.cast(std, dtype=dtype)
+        avg = at.cast(avg, dtype=dtype)
+        std = at.cast(std, dtype=dtype)
 
         # generate even number of uniform samples
         # Do manual constant folding to lower optiimizer work.
@@ -1205,14 +1204,14 @@ class MRG_RandomStream:
             # use valid samples
             to_fix0 = (z0 < -2.0) | (z0 > 2.0)
             to_fix1 = (z1 < -2.0) | (z1 > 2.0)
-            z0_valid = z0[aet.nonzero(~to_fix0)]
-            z1_valid = z1[aet.nonzero(~to_fix1)]
+            z0_valid = z0[at.nonzero(~to_fix0)]
+            z1_valid = z1[at.nonzero(~to_fix1)]
 
             # re-sample invalid samples
-            to_fix0 = aet.nonzero(to_fix0)[0]
-            to_fix1 = aet.nonzero(to_fix1)[0]
+            to_fix0 = at.nonzero(to_fix0)[0]
+            to_fix1 = at.nonzero(to_fix1)[0]
             n_fix_samples = to_fix0.size + to_fix1.size
-            lower = aet.constant(1.0 / np.e ** 2, dtype=dtype)
+            lower = at.constant(1.0 / np.e**2, dtype=dtype)
             u_fix = self.uniform(
                 (n_fix_samples,),
                 low=lower,
@@ -1227,9 +1226,9 @@ class MRG_RandomStream:
             z1_fixed = r_fix[to_fix0.size :] * sin_theta[to_fix1]
 
             # pack everything together to a useful result
-            norm_samples = aet.join(0, z0_valid, z0_fixed, z1_valid, z1_fixed)
+            norm_samples = at.join(0, z0_valid, z0_fixed, z1_valid, z1_fixed)
         else:
-            norm_samples = aet.join(0, z0, z1)
+            norm_samples = at.join(0, z0, z1)
         if isinstance(n_odd_samples, Variable):
             samples = norm_samples[:n_odd_samples]
         elif n_odd_samples % 2 == 1:
@@ -1275,7 +1274,7 @@ class MRG_RandomStream:
         normal
         """
         # constant taken from scipy.stats.truncnorm.std(a=-2, b=2, loc=0., scale=1.)
-        std = std / aet.constant(0.87962566103423978)
+        std = std / at.constant(0.87962566103423978)
         return self.normal(
             size=size,
             avg=avg,
@@ -1314,13 +1313,13 @@ def _check_size(size):
         if size.ndim == 1:
             return size
         elif size.ndim == 0:
-            return aet.stack([size], ndim=1)
+            return at.stack([size], ndim=1)
         else:
             raise ValueError(
                 "Aesara variable must have 1 dimension to be a valid size.", size
             )
     elif isinstance(size, (np.integer, int)):
-        return aet.constant([size], ndim=1)
+        return at.constant([size], ndim=1)
     elif not isinstance(size, (tuple, list)):
         raise ValueError("Size must be a int, tuple, list or Aesara variable.", size)
 
@@ -1341,7 +1340,7 @@ def _check_size(size):
                 i,
             )
 
-    return aet.as_tensor_variable(size, ndim=1)
+    return at.as_tensor_variable(size, ndim=1)
 
 
 @local_optimizer((mrg_uniform_base,))
@@ -1358,7 +1357,7 @@ def mrg_random_make_inplace(fgraph, node):
 optdb.register(
     "random_make_inplace_mrg",
     in2out(mrg_random_make_inplace, ignore_newtrees=True),
-    99,
     "fast_run",
     "inplace",
+    position=99,
 )
