@@ -2,21 +2,20 @@
 
 import copy
 import dataclasses
-import logging
 from sys import maxsize
-from typing import Dict, List, Tuple
+from typing import Dict, List, Optional, Tuple
 
 import numpy as np
 
 import aesara
 from aesara import scalar as aes
-from aesara import tensor as aet
+from aesara import tensor as at
 from aesara.compile import optdb
 from aesara.compile.function.types import deep_copy_op
 from aesara.configdefaults import config
 from aesara.graph.basic import (
+    Apply,
     Constant,
-    Node,
     Variable,
     clone_replace,
     equal_computations,
@@ -26,10 +25,12 @@ from aesara.graph.basic import (
 )
 from aesara.graph.destroyhandler import DestroyHandler
 from aesara.graph.features import ReplaceValidate
-from aesara.graph.fg import FunctionGraph, InconsistencyError
+from aesara.graph.fg import FunctionGraph
 from aesara.graph.op import compute_test_value
 from aesara.graph.opt import GlobalOptimizer, in2out, local_optimizer
 from aesara.graph.optdb import EquilibriumDB, SequenceDB
+from aesara.graph.type import HasShape
+from aesara.graph.utils import InconsistencyError
 from aesara.scan.op import Scan, ScanInfo
 from aesara.scan.utils import (
     ScanArgs,
@@ -56,8 +57,6 @@ from aesara.tensor.subtensor import (
 from aesara.tensor.var import TensorConstant, get_unique_value
 
 
-_logger = logging.getLogger("aesara.scan.opt")
-
 list_opt_slice = [
     math_opt.local_abs_merge,
     math_opt.local_mul_switch_sink,
@@ -124,7 +123,7 @@ def remove_constants_and_unused_inputs_scan(fgraph, node):
             try:
                 # This works if input is a constant that has all entries
                 # equal
-                givens[op_ins[idx]] = node_inp.clone()[0]
+                givens[op_ins[idx]] = node_inp[0]
             except TypeError:
                 pass
         elif op_ins[idx] in all_ins:
@@ -149,7 +148,7 @@ def remove_constants_and_unused_inputs_scan(fgraph, node):
     nw_outer_nonseq = []
     for idx, (nw_in, nw_out) in enumerate(zip(non_seqs, outer_non_seqs)):
         if isinstance(nw_out, Constant):
-            givens[nw_in] = nw_out.clone()
+            givens[nw_in] = nw_out
         elif nw_in in all_ins:
             # Indices of elements of nw_outer_nonseq that are equivalent
             # to nw_out.
@@ -175,7 +174,6 @@ def remove_constants_and_unused_inputs_scan(fgraph, node):
             op_outs,
             nw_info,
             mode=op.mode,
-            gpua=op.gpua,
             as_while=op.as_while,
             profile=op.profile,
             truncate_gradient=op.truncate_gradient,
@@ -202,12 +200,11 @@ def push_out_non_seq_scan(fgraph, node):
     if not isinstance(node.op, Scan):
         return False
 
-    # this flag tells if there was any change during the last iterations
-    clean_inputs, clean_outputs = reconstruct_graph(node.op.inputs, node.op.outputs)
+    node_inputs, node_outputs = node.op.inputs, node.op.outputs
 
-    local_fgraph_topo = io_toposort(clean_inputs, clean_outputs)
-    local_fgraph_outs_set = set(clean_outputs)
-    local_fgraph_outs_map = {v: k for k, v in enumerate(clean_outputs)}
+    local_fgraph_topo = io_toposort(node_inputs, node_outputs)
+    local_fgraph_outs_set = set(node_outputs)
+    local_fgraph_outs_map = {v: k for k, v in enumerate(node_outputs)}
 
     to_remove_set = set()
     to_replace_set = set()
@@ -220,18 +217,21 @@ def push_out_non_seq_scan(fgraph, node):
 
     add_to_replace.n = 0
 
+    # The variables that will replace the variables pushed-out of the
+    # inner-graph
     replace_with_in = []
+    # The variables that have been pushed-out of the graph
     replace_with_out = []
 
     op = node.op
     # Construct the list of non_sequences to simplify a few things
-    inner_non_seqs = op.inner_non_seqs(clean_inputs)
+    inner_non_seqs = op.inner_non_seqs(node_inputs)
     inner_non_seqs_set = set(inner_non_seqs)
     inner_non_seqs_map = {v: k for k, v in enumerate(inner_non_seqs)}
 
     outer_non_seqs = op.outer_non_seqs(node.inputs)
 
-    inner_seqs = op.inner_seqs(clean_inputs)
+    inner_seqs = op.inner_seqs(node_inputs)
     outer_seqs = op.outer_seqs(node.inputs)
 
     assert len(inner_non_seqs) == len(outer_non_seqs)
@@ -241,55 +241,60 @@ def push_out_non_seq_scan(fgraph, node):
         if (  # we haven't already looked at this node
             nd not in to_remove_set
             and all(
-                [
-                    (
-                        (x in inner_non_seqs_set)
-                        or (x.owner in to_remove_set)
-                        or isinstance(x, Constant)
-                    )
-                    for x in nd.inputs
-                ]
+                (
+                    (x in inner_non_seqs_set)
+                    or (x.owner in to_remove_set)
+                    or isinstance(x, Constant)
+                )
+                for x in nd.inputs
             )
-            and
-            # we can do this because the assumption is that a
-            # viewOp or deepCopyOp will be just at the end of the
-            # function and not somewhere in the middle ..
-            not isinstance(nd.op, aesara.compile.ViewOp)
+            # We can (supposedly) do this because the assumption is that a
+            # `ViewOp` or `DeepCopyOp` will be just at the end of the
+            # function and not somewhere in the middle
+            and not isinstance(nd.op, aesara.compile.ViewOp)
             and not isinstance(nd.op, aesara.compile.DeepCopyOp)
         ):
+            # We have a candidate node to remove from the inner-graph
 
-            # We have a candidate node to removable
-            # Step 1. Reconstruct it on outside
+            # Step 1. Reconstruct the node using the relevant outer-inputs.
+            #
+            # More specifically, the node's current inputs are either
+            # a) inner-graph input place-holders for non-sequences,
+            # b) the outputs of other nodes being pushed out of the inner-graph,
+            # c) or constants.
             to_remove_set.add(nd)
-            outside_ins = []
-            for x in nd.inputs:
-                if x in inner_non_seqs_set:
-                    _idx = inner_non_seqs_map[x]
-                    outside_ins.append(outer_non_seqs[_idx])
-                elif x in to_replace_set:
-                    outside_ins.append(replace_with_out[to_replace_map[x]])
-                elif isinstance(x, Constant):
-                    outside_ins.append(x.clone())
+            new_inputs = []
+            for old_input in nd.inputs:
+                if old_input in inner_non_seqs_set:
+                    # This is case a), so we want to use the corresponding
+                    # outer-graph input as the input to our new pushed-out node
+                    _idx = inner_non_seqs_map[old_input]
+                    new_input = outer_non_seqs[_idx]
+                elif old_input in to_replace_set:
+                    # This is case b), so we want to use the new pushed-out node
+                    # as the input to this new pushed-out node
+                    new_input = replace_with_out[to_replace_map[old_input]]
                 else:
-                    # TODO: Explain why is this an error, and raise an
-                    # appropriate exception type.
-                    raise RuntimeError()
-            outside_ins = [
-                x.type.filter_variable(y) for x, y in zip(nd.inputs, outside_ins)
-            ]
+                    assert isinstance(old_input, Constant)
+                    new_input = old_input
 
-            nw_outer_node = nd.op.make_node(*outside_ins)
+                new_input = old_input.type.filter_variable(new_input)
+                new_inputs.append(new_input)
+
+            pushed_out_node = nd.op.make_node(*new_inputs)
 
             if config.compute_test_value != "off":
-                compute_test_value(nw_outer_node)
+                compute_test_value(pushed_out_node)
 
-            # Step 2. Create variables for replacements
+            # Step 2. Create variables to replace the old outputs of the node
+            # that we're pushing out of the inner-graph
             for idx, y in enumerate(nd.outputs):
-                y_place_holder = safe_new(y, "_replace")
+                y_place_holder = y.clone()
+                # y_place_holder = safe_new(y, "_replace")
                 add_to_replace(y)
                 replace_with_in.append(y_place_holder)
-                assert isinstance(y, type(nw_outer_node.outputs[idx]))
-                replace_with_out.append(nw_outer_node.outputs[idx])
+                assert isinstance(y, type(pushed_out_node.outputs[idx]))
+                replace_with_out.append(pushed_out_node.outputs[idx])
 
     # We need to check all candidate replacements and choose those that
     # make sense for us
@@ -308,7 +313,7 @@ def push_out_non_seq_scan(fgraph, node):
     for out, idx in to_replace_map.items():
         if (  # If types are different, conversion Op will be inserted,
             # and it may trigger an infinite loop.
-            replace_with_in[idx].type == out.type
+            out.type.is_super(replace_with_in[idx].type)
             and out in to_keep_set
             and out.owner not in existent_nodes_set
         ):
@@ -325,14 +330,14 @@ def push_out_non_seq_scan(fgraph, node):
             clean_to_replace, clean_replace_with_in, clean_replace_with_out
         ):
             if isinstance(repl_out, Constant):
-                repl_in = repl_out.clone()
+                repl_in = repl_out
             else:
                 nw_inner.append(repl_in)
                 nw_outer.append(repl_out)
             givens[to_repl] = repl_in
 
-        op_outs = clone_replace(clean_outputs, replace=givens)
-        op_ins = clean_inputs + nw_inner
+        op_outs = clone_replace(node_outputs, replace=givens)
+        op_ins = node_inputs + nw_inner
 
         # Reconstruct node
         nwScan = Scan(
@@ -340,7 +345,6 @@ def push_out_non_seq_scan(fgraph, node):
             op_outs,
             op.info,
             mode=op.mode,
-            gpua=op.gpua,
             as_while=op.as_while,
             profile=op.profile,
             truncate_gradient=op.truncate_gradient,
@@ -363,7 +367,7 @@ def push_out_non_seq_scan(fgraph, node):
                 x = node.outputs[local_fgraph_outs_map[out]]
                 y = replace_with_out[idx]
                 y_shape = [shp for shp in y.shape]
-                replace_with[x] = aet.alloc(y, node.inputs[0], *y_shape)
+                replace_with[x] = at.alloc(y, node.inputs[0], *y_shape)
 
         # We need to add one extra dimension to the outputs
         # because the scan op expects for a tensor3, to which an
@@ -392,8 +396,8 @@ def push_out_non_seq_scan(fgraph, node):
 def push_out_seq_scan(fgraph, node):
     r"""Push out the variables inside the `Scan` that depend only on constants and sequences.
 
-    This optimization resembles `PushOutNonSeqScan` but it tries to push, out of
-    the inner function, the computation that only relies on sequence and
+    This optimization resembles `push_out_non_seq_scan` but it tries to push--out of
+    the inner function--the computation that only relies on sequence and
     non-sequence inputs. The idea behind this optimization is that, when it is
     possible to do so, it is generally more computationally efficient to perform
     a single operation on a large tensor rather then perform that same operation
@@ -403,12 +407,11 @@ def push_out_seq_scan(fgraph, node):
     if not isinstance(node.op, Scan):
         return False
 
-    # this flag tells if there was any change during the last iterations
-    clean_inputs, clean_outputs = reconstruct_graph(node.op.inputs, node.op.outputs)
+    node_inputs, node_outputs = node.op.inputs, node.op.outputs
 
-    local_fgraph_topo = io_toposort(clean_inputs, clean_outputs)
-    local_fgraph_outs_set = set(clean_outputs)
-    local_fgraph_outs_map = {v: k for k, v in enumerate(clean_outputs)}
+    local_fgraph_topo = io_toposort(node_inputs, node_outputs)
+    local_fgraph_outs_set = set(node_outputs)
+    local_fgraph_outs_map = {v: k for k, v in enumerate(node_outputs)}
 
     to_remove_set = set()
     to_replace_set = set()
@@ -426,12 +429,12 @@ def push_out_seq_scan(fgraph, node):
 
     op = node.op
     # Construct the list of non_sequences to simplify a few things
-    inner_non_seqs = op.inner_non_seqs(clean_inputs)
+    inner_non_seqs = op.inner_non_seqs(node_inputs)
     inner_non_seqs_set = set(inner_non_seqs)
     inner_non_seqs_map = {v: k for k, v in enumerate(inner_non_seqs)}
 
     outer_non_seqs = op.outer_non_seqs(node.inputs)
-    inner_seqs = op.inner_seqs(clean_inputs)
+    inner_seqs = op.inner_seqs(node_inputs)
     inner_seqs_set = set(inner_seqs)
     inner_seqs_map = {v: k for k, v in enumerate(inner_seqs)}
 
@@ -460,26 +463,18 @@ def push_out_seq_scan(fgraph, node):
             for x in nd.inputs:
                 if x in inner_non_seqs_set:
                     _idx = inner_non_seqs_map[x]
-                    outside_ins.append(outer_non_seqs[_idx])
+                    new_input = outer_non_seqs[_idx]
                 elif x in inner_seqs_set:
-                    outside_ins.append(outer_seqs[inner_seqs_map[x]])
+                    new_input = outer_seqs[inner_seqs_map[x]]
                     depends_on_seqs = True
                 elif x in to_replace_set:
-                    outside_ins.append(replace_with_out[to_replace_map[x]])
+                    new_input = replace_with_out[to_replace_map[x]]
                     depends_on_seqs = True
-                elif isinstance(x, Constant):
-                    outside_ins.append(x.clone())
                 else:
-                    raise Exception(
-                        (
-                            "Error in the `scan_pushout_seq_"
-                            "operations`. The optimization tries "
-                            "to move some computation from scan "
-                            "which is not allowed to move. Report "
-                            "this on aesara-users list"
-                        ),
-                        x,
-                    )
+                    assert isinstance(x, Constant)
+                    new_input = x
+
+                outside_ins.append(new_input)
 
             if not depends_on_seqs:
                 # Removing this node from the inner graph of scan
@@ -557,7 +552,7 @@ def push_out_seq_scan(fgraph, node):
             and
             # If types are different, conversion Op will be inserted,
             # and it may trigger an infinite loop.
-            replace_with_in[idx].type == out.type
+            out.type.is_super(replace_with_in[idx].type)
         ):
 
             clean_to_replace.append(out)
@@ -573,15 +568,15 @@ def push_out_seq_scan(fgraph, node):
             clean_to_replace, clean_replace_with_in, clean_replace_with_out
         ):
             if isinstance(repl_out, Constant):
-                repl_in = repl_out.clone()
+                repl_in = repl_out
             else:
                 nw_inner.append(repl_in)
                 nw_outer.append(repl_out)
 
             givens[to_repl] = repl_in
 
-        op_outs = clone_replace(clean_outputs, replace=givens)
-        op_ins = nw_inner + clean_inputs
+        op_outs = clone_replace(node_outputs, replace=givens)
+        op_ins = nw_inner + node_inputs
 
         # Reconstruct node
         nw_info = dataclasses.replace(op.info, n_seqs=op.info.n_seqs + len(nw_inner))
@@ -590,7 +585,6 @@ def push_out_seq_scan(fgraph, node):
             op_outs,
             nw_info,
             mode=op.mode,
-            gpua=op.gpua,
             as_while=op.as_while,
             profile=op.profile,
             truncate_gradient=op.truncate_gradient,
@@ -615,7 +609,7 @@ def push_out_seq_scan(fgraph, node):
             if out in local_fgraph_outs_set:
                 x = node.outputs[local_fgraph_outs_map[out]]
                 _y = replace_with_out[idx]
-                ls = clean_outputs
+                ls = node_outputs
                 if out in op.inner_mitsot_outs(ls):
                     odx = op.inner_mitsot_outs(ls).index(out)
                     inp = op.outer_mitsot(node.inputs)[odx]
@@ -654,9 +648,9 @@ def inner_sitsot_only_last_step_used(
 
     if len(fgraph.clients[outer_var]) == 1:
         client = fgraph.clients[outer_var][0][0]
-        if client != "output" and isinstance(client.op, Subtensor):
+        if isinstance(client, Apply) and isinstance(client.op, Subtensor):
             lst = get_idx_list(client.inputs, client.op.idx_list)
-            if len(lst) == 1 and aet.extract_constant(lst[0]) == -1:
+            if len(lst) == 1 and at.extract_constant(lst[0]) == -1:
                 return True
 
     return False
@@ -664,10 +658,12 @@ def inner_sitsot_only_last_step_used(
 
 def get_outer_ndim(var: Variable, scan_args: ScanArgs) -> int:
     """Determine the number of dimension a variable would have if it was pushed out of a `Scan`."""
+    assert isinstance(var.type, HasShape)
+
     if var in scan_args.inner_in_non_seqs or isinstance(var, Constant):
-        outer_ndim = var.ndim
+        outer_ndim = var.type.ndim
     else:
-        outer_ndim = var.ndim + 1
+        outer_ndim = var.type.ndim + 1
 
     return outer_ndim
 
@@ -675,14 +671,14 @@ def get_outer_ndim(var: Variable, scan_args: ScanArgs) -> int:
 def push_out_inner_vars(
     fgraph: FunctionGraph,
     inner_vars: List[Variable],
-    old_scan_node: Node,
+    old_scan_node: Apply,
     old_scan_args: ScanArgs,
 ) -> Tuple[List[Variable], ScanArgs, Dict[Variable, Variable]]:
 
-    outer_vars = [None] * len(inner_vars)
+    tmp_outer_vars: List[Optional[Variable]] = []
     new_scan_node = old_scan_node
     new_scan_args = old_scan_args
-    replacements = {}
+    replacements: Dict[Variable, Variable] = {}
 
     # For the inner_vars that already exist in the outer graph,
     # simply obtain a reference to them
@@ -690,26 +686,32 @@ def push_out_inner_vars(
 
         var = inner_vars[idx]
 
+        new_outer_var: Optional[Variable] = None
+
         if var in old_scan_args.inner_in_seqs:
             idx_seq = old_scan_args.inner_in_seqs.index(var)
-            outer_vars[idx] = old_scan_args.outer_in_seqs[idx_seq]
+            new_outer_var = old_scan_args.outer_in_seqs[idx_seq]
 
         elif var in old_scan_args.inner_in_non_seqs:
             idx_non_seq = old_scan_args.inner_in_non_seqs.index(var)
-            outer_vars[idx] = old_scan_args.outer_in_non_seqs[idx_non_seq]
+            new_outer_var = old_scan_args.outer_in_non_seqs[idx_non_seq]
 
         elif isinstance(var, Constant):
-            outer_vars[idx] = var.clone()
+            new_outer_var = var
 
         elif var in old_scan_args.inner_out_nit_sot:
             idx_nitsot = old_scan_args.inner_out_nit_sot.index(var)
-            outer_vars[idx] = old_scan_args.outer_out_nit_sot[idx_nitsot]
+            new_outer_var = old_scan_args.outer_out_nit_sot[idx_nitsot]
+
+        tmp_outer_vars.append(new_outer_var)
 
     # For the inner_vars that don't already exist in the outer graph, add
     # them as new nitsot outputs to the scan node.
-    idx_add_as_nitsots = [i for i in range(len(outer_vars)) if outer_vars[i] is None]
+    idx_add_as_nitsots = [i for i, v in enumerate(tmp_outer_vars) if v is None]
     add_as_nitsots = [inner_vars[idx] for idx in idx_add_as_nitsots]
 
+    new_outs: List[Variable] = []
+
     if len(add_as_nitsots) > 0:
 
         new_scan_node, replacements = add_nitsot_outputs(
@@ -726,18 +728,25 @@ def push_out_inner_vars(
         )
 
         new_outs = new_scan_args.outer_out_nit_sot[-len(add_as_nitsots) :]
-        for i in range(len(new_outs)):
-            outer_vars[idx_add_as_nitsots[i]] = new_outs[i]
+
+    outer_vars: List[Variable] = []
+
+    for i, v in enumerate(tmp_outer_vars):
+        if i in idx_add_as_nitsots:
+            outer_vars.append(new_outs.pop(0))
+        else:
+            assert v is not None
+            outer_vars.append(v)
 
     return outer_vars, new_scan_args, replacements
 
 
 def add_nitsot_outputs(
     fgraph: FunctionGraph,
-    old_scan_node: Node,
+    old_scan_node: Apply,
     old_scan_args: ScanArgs,
     new_outputs_inner,
-) -> Tuple[Node, Dict[Variable, Variable]]:
+) -> Tuple[Apply, Dict[Variable, Variable]]:
 
     nb_new_outs = len(new_outputs_inner)
 
@@ -757,7 +766,6 @@ def add_nitsot_outputs(
         new_scan_args.inner_outputs,
         new_scan_args.info,
         mode=old_scan_node.op.mode,
-        gpua=old_scan_node.op.gpua,
         as_while=old_scan_node.op.as_while,
         profile=old_scan_node.op.profile,
         truncate_gradient=old_scan_node.op.truncate_gradient,
@@ -767,7 +775,10 @@ def add_nitsot_outputs(
     )
 
     # Create the Apply node for the scan op
-    new_scan_node = new_scan_op(*new_scan_args.outer_inputs, return_list=True)[0].owner
+    new_scan_outs = new_scan_op(*new_scan_args.outer_inputs, return_list=True)
+    assert isinstance(new_scan_outs, list)
+    new_scan_node = new_scan_outs[0].owner
+    assert new_scan_node is not None
 
     # Modify the outer graph to make sure the outputs of the new scan are
     # used instead of the outputs of the old scan
@@ -784,7 +795,7 @@ def add_nitsot_outputs(
     # replacements = dict(zip(old_scan_node.outputs, new_node_old_outputs))
     # replacements["remove"] = [old_scan_node]
     # return new_scan_node, replacements
-    fgraph.replace_all_validate_remove(
+    fgraph.replace_all_validate_remove(  # type: ignore
         list(zip(old_scan_node.outputs, new_node_old_outputs)),
         remove=[old_scan_node],
         reason="scan_pushout_add",
@@ -871,7 +882,7 @@ def push_out_add_scan(fgraph, node):
                     # so that they become matrices. This is because a
                     # dot is usually faster on two large matrices than
                     # a bunch of small ones
-                    outer_dot_inputs[0] = aet.flatten(
+                    outer_dot_inputs[0] = at.flatten(
                         outer_dot_inputs[0].dimshuffle(1, 0, 2), ndim=2
                     )
 
@@ -908,10 +919,9 @@ class ScanInplaceOptimizer(GlobalOptimizer):
 
     """
 
-    def __init__(self, typeInfer=None, gpua_flag=False):
+    def __init__(self, typeInfer=None):
         super().__init__()
         self.typeInfer = typeInfer
-        self.gpua_flag = gpua_flag
 
     def add_requirements(self, fgraph):
         fgraph.attach_feature(ReplaceValidate())
@@ -983,7 +993,6 @@ class ScanInplaceOptimizer(GlobalOptimizer):
             op.info,
             mode=op.mode,
             typeConstructor=typeConstructor,
-            gpua=op.gpua,
             as_while=op.as_while,
             profile=op.profile,
             truncate_gradient=op.truncate_gradient,
@@ -1015,9 +1024,7 @@ class ScanInplaceOptimizer(GlobalOptimizer):
 
         alloc_ops = (Alloc, AllocEmpty)
         nodes = fgraph.toposort()[::-1]
-        scan_nodes = [
-            x for x in nodes if (isinstance(x.op, Scan) and x.op.gpua == self.gpua_flag)
-        ]
+        scan_nodes = [x for x in nodes if (isinstance(x.op, Scan))]
         for scan_idx in range(len(scan_nodes)):
 
             # First attempt to make the Scan compute inplace every recurrent
@@ -1096,7 +1103,7 @@ def sanitize(x):
     if x is None:
         return None
     else:
-        return aet.as_tensor_variable(x)
+        return at.as_tensor_variable(x)
 
 
 @local_optimizer([Scan])
@@ -1200,7 +1207,7 @@ def save_mem_new_scan(fgraph, node):
 
             # 2.1 outputs of the function
             # => output needs all its intermediate values
-            if type(cl) == str:
+            if isinstance(cl, str):
                 # if the node is actually an output, then
                 # we need to store the entire thing
                 global_nsteps = None
@@ -1241,9 +1248,9 @@ def save_mem_new_scan(fgraph, node):
                 if isinstance(this_slice[0], slice) and this_slice[0].stop is None:
                     global_nsteps = None
                 if isinstance(cf_slice[0], slice):
-                    stop = aet.extract_constant(cf_slice[0].stop)
+                    stop = at.extract_constant(cf_slice[0].stop)
                 else:
-                    stop = aet.extract_constant(cf_slice[0]) + 1
+                    stop = at.extract_constant(cf_slice[0]) + 1
                 if stop == maxsize or stop == length:
                     stop = None
                 else:
@@ -1263,13 +1270,13 @@ def save_mem_new_scan(fgraph, node):
                     if isinstance(stop, Variable):
                         global_nsteps["sym"] += [stop]
                     # not if it is maxsize
-                    elif type(stop) == int and stop == maxsize:
+                    elif isinstance(stop, int) and stop == maxsize:
                         global_nsteps = None
                     # yes if it is a int k, 0 < k < maxsize
-                    elif type(stop) == int and global_nsteps["real"] < stop:
+                    elif isinstance(stop, int) and global_nsteps["real"] < stop:
                         global_nsteps["real"] = stop
                     # yes if it is a int k, 0 < k < maxsize
-                    elif type(stop) == int and stop > 0:
+                    elif isinstance(stop, int) and stop > 0:
                         pass
                     # not otherwise
                     else:
@@ -1311,7 +1318,7 @@ def save_mem_new_scan(fgraph, node):
     for i, out in enumerate(node.outputs[:c_outs]):
         # look at all its clients
         for cl, _ in fgraph.clients[out]:
-            if type(cl) == str:
+            if isinstance(cl, str):
                 store_steps[i] = 0
                 break
             elif not isinstance(cl.op, Subtensor):
@@ -1337,9 +1344,9 @@ def save_mem_new_scan(fgraph, node):
                 cf_slice = get_canonical_form_slice(this_slice[0], length)
 
                 if isinstance(cf_slice[0], slice):
-                    start = aet.extract_constant(cf_slice[0].start)
+                    start = at.extract_constant(cf_slice[0].start)
                 else:
-                    start = aet.extract_constant(cf_slice[0])
+                    start = at.extract_constant(cf_slice[0])
                 if start == 0 or store_steps[i] == 0:
                     store_steps[i] = 0
                 else:
@@ -1419,13 +1426,13 @@ def save_mem_new_scan(fgraph, node):
                             nw_inputs[offset + idx].owner.op, IncSubtensor
                         )
                         _nw_input = nw_inputs[offset + idx].owner.inputs[1]
-                        cval = aet.as_tensor_variable(val)
-                        initl = aet.as_tensor_variable(init_l[i])
-                        tmp_idx = aet.switch(cval < initl, cval + initl, cval - initl)
+                        cval = at.as_tensor_variable(val)
+                        initl = at.as_tensor_variable(init_l[i])
+                        tmp_idx = at.switch(cval < initl, cval + initl, cval - initl)
                         nw_input = expand_empty(_nw_input, tmp_idx)
                     else:
-                        tmp = aet.as_tensor_variable(val)
-                        initl = aet.as_tensor_variable(init_l[i])
+                        tmp = at.as_tensor_variable(val)
+                        initl = at.as_tensor_variable(init_l[i])
                         tmp = maximum(tmp, initl)
                         nw_input = nw_inputs[offset + idx][:tmp]
 
@@ -1505,7 +1512,7 @@ def save_mem_new_scan(fgraph, node):
         # 3.6 Compose the new scan
         # TODO: currently we don't support scan with 0 step. So
         # don't create one.
-        if aet.extract_constant(node_ins[0]) == 0:
+        if at.extract_constant(node_ins[0]) == 0:
             return False
 
         # Do not call make_node for test_value
@@ -1514,7 +1521,6 @@ def save_mem_new_scan(fgraph, node):
             outs,
             info,
             mode=op.mode,
-            gpua=op.gpua,
             as_while=op.as_while,
             profile=op.profile,
             truncate_gradient=op.truncate_gradient,
@@ -1811,7 +1817,6 @@ class ScanMerge(GlobalOptimizer):
             truncate_gradient=old_op.truncate_gradient,
             allow_gc=old_op.allow_gc,
             name="&".join([nd.op.name for nd in nodes]),
-            gpua=False,
             as_while=as_while,
         )
         new_outs = new_op(*outer_ins)
@@ -1988,7 +1993,6 @@ def scan_merge_inouts(fgraph, node):
             inner_outputs,
             info,
             mode=node.op.mode,
-            gpua=node.op.gpua,
             as_while=node.op.as_while,
             profile=node.op.profile,
             truncate_gradient=node.op.truncate_gradient,
@@ -2254,7 +2258,6 @@ def push_out_dot1_scan(fgraph, node):
                         new_inner_outs,
                         new_info,
                         mode=op.mode,
-                        gpua=op.gpua,
                         as_while=op.as_while,
                         profile=op.profile,
                         truncate_gradient=op.truncate_gradient,
@@ -2275,7 +2278,7 @@ def push_out_dot1_scan(fgraph, node):
                     )
 
                     new_outs = new_op(*_scan_inputs)
-                    if type(new_outs) not in (list, tuple):
+                    if not isinstance(new_outs, (list, tuple)):
                         new_outs = [new_outs]
 
                     # We need now to pair correctly the new outputs
@@ -2336,68 +2339,68 @@ scan_eqopt2 = EquilibriumDB()
 # scan_eqopt1 before ShapeOpt at 0.1
 # This is needed to don't have ShapeFeature trac old Scan that we
 # don't want to reintroduce.
-optdb.register("scan_eqopt1", scan_eqopt1, 0.05, "fast_run", "scan")
+optdb.register("scan_eqopt1", scan_eqopt1, "fast_run", "scan", position=0.05)
 # We run before blas opt at 1.7 and specialize 2.0
 # but after stabilize at 1.5. Should we put it before stabilize?
-optdb.register("scan_eqopt2", scan_eqopt2, 1.6, "fast_run", "scan")
+optdb.register("scan_eqopt2", scan_eqopt2, "fast_run", "scan", position=1.6)
 # ScanSaveMem should execute only once per node.
 optdb.register(
     "scan_save_mem",
     in2out(save_mem_new_scan, ignore_newtrees=True),
-    1.61,
     "fast_run",
     "scan",
+    position=1.61,
 )
 optdb.register(
     "scan_make_inplace",
     ScanInplaceOptimizer(typeInfer=None),
-    75,
     "fast_run",
     "inplace",
     "scan",
+    position=75,
 )
 
-scan_eqopt1.register("all_pushout_opt", scan_seqopt1, 1, "fast_run", "scan")
+scan_eqopt1.register("all_pushout_opt", scan_seqopt1, "fast_run", "scan", position=1)
 
 
 scan_seqopt1.register(
     "scan_remove_constants_and_unused_inputs0",
     in2out(remove_constants_and_unused_inputs_scan, ignore_newtrees=True),
-    1,
     "remove_constants_and_unused_inputs_scan",
     "fast_run",
     "scan",
+    position=1,
 )
 
 
 scan_seqopt1.register(
     "scan_pushout_nonseqs_ops",
     in2out(push_out_non_seq_scan, ignore_newtrees=True),
-    2,
     "fast_run",
     "scan",
     "scan_pushout",
+    position=2,
 )
 
 
 scan_seqopt1.register(
     "scan_pushout_seqs_ops",
     in2out(push_out_seq_scan, ignore_newtrees=True),
-    3,
     "fast_run",
     "scan",
     "scan_pushout",
+    position=3,
 )
 
 
 scan_seqopt1.register(
     "scan_pushout_dot1",
     in2out(push_out_dot1_scan, ignore_newtrees=True),
-    4,
     "fast_run",
     "more_mem",
     "scan",
     "scan_pushout",
+    position=4,
 )
 
 
@@ -2405,62 +2408,62 @@ scan_seqopt1.register(
     "scan_pushout_add",
     # TODO: Perhaps this should be an `EquilibriumOptimizer`?
     in2out(push_out_add_scan, ignore_newtrees=False),
-    5,
     "fast_run",
     "more_mem",
     "scan",
     "scan_pushout",
+    position=5,
 )
 
 
 scan_eqopt2.register(
     "constant_folding_for_scan2",
     in2out(basic_opt.constant_folding, ignore_newtrees=True),
-    1,
     "fast_run",
     "scan",
+    position=1,
 )
 
 
 scan_eqopt2.register(
     "scan_remove_constants_and_unused_inputs1",
     in2out(remove_constants_and_unused_inputs_scan, ignore_newtrees=True),
-    2,
     "remove_constants_and_unused_inputs_scan",
     "fast_run",
     "scan",
+    position=2,
 )
 
 
 # after const merge but before stabilize so that we can have identity
 # for equivalent nodes but we still have the chance to hoist stuff out
 # of the scan later.
-scan_eqopt2.register("scan_merge", ScanMerge(), 4, "fast_run", "scan")
+scan_eqopt2.register("scan_merge", ScanMerge(), "fast_run", "scan", position=4)
 
 # After Merge optimization
 scan_eqopt2.register(
     "scan_remove_constants_and_unused_inputs2",
     in2out(remove_constants_and_unused_inputs_scan, ignore_newtrees=True),
-    5,
     "remove_constants_and_unused_inputs_scan",
     "fast_run",
     "scan",
+    position=5,
 )
 
 scan_eqopt2.register(
     "scan_merge_inouts",
     in2out(scan_merge_inouts, ignore_newtrees=True),
-    6,
     "fast_run",
     "scan",
+    position=6,
 )
 
 # After everything else
 scan_eqopt2.register(
     "scan_remove_constants_and_unused_inputs3",
     in2out(remove_constants_and_unused_inputs_scan, ignore_newtrees=True),
-    8,
     "remove_constants_and_unused_inputs_scan",
     "fast_run",
     "scan",
+    position=8,
 )
