@@ -8,15 +8,15 @@ from aesara.configdefaults import config
 from aesara.gradient import DisconnectedType
 from aesara.graph.basic import Apply
 from aesara.graph.null_type import NullType
-from aesara.graph.op import COp, ExternalCOp, OpenMPOp
-from aesara.graph.params_type import ParamsType
 from aesara.graph.utils import MethodNotDefined
 from aesara.link.c.basic import failure_code
+from aesara.link.c.op import COp, ExternalCOp, OpenMPOp
+from aesara.link.c.params_type import ParamsType
 from aesara.misc.frozendict import frozendict
 from aesara.misc.safe_asarray import _asarray
 from aesara.printing import FunctionPrinter, Printer, pprint
 from aesara.scalar import get_scalar_type
-from aesara.scalar.basic import Scalar
+from aesara.scalar.basic import ScalarType
 from aesara.scalar.basic import bool as scalar_bool
 from aesara.scalar.basic import identity as scalar_identity
 from aesara.scalar.basic import transfer_type, upcast
@@ -474,9 +474,9 @@ second dimension
             if self.inplace_pattern:
                 items = list(self.inplace_pattern.items())
                 items.sort()
-                return f"Elemwise{{{self.scalar_op}}}{items}"
+                return f"{type(self).__name__}{{{self.scalar_op}}}{items}"
             else:
-                return "Elemwise{%s}" % (self.scalar_op)
+                return f"{type(self).__name__}{{{self.scalar_op}}}"
         else:
             return self.name
 
@@ -815,7 +815,7 @@ second dimension
                     # there must be some input that is not broadcastable in
                     # dimension 'dim'
                     for ishp, i in zip(i_shapes, node.inputs):
-                        if isinstance(i.type, Scalar):
+                        if isinstance(i.type, ScalarType):
                             continue  # we skip scalar
                         if not i.type.broadcastable[dim]:
                             # input i is not broadcastable in position dim
@@ -1298,21 +1298,17 @@ class CAReduce(COp):
         inp_dims = input.type.ndim
         inp_bdcast = input.type.broadcastable
         inp_dtype = input.type.dtype
-        copy_op = False
 
         axis = self.axis
         if axis is None:
-            axis = list(range(len(inp_bdcast)))
+            axis = list(range(inp_dims))
 
-        axis = list(axis)
-        for i, a in enumerate(axis):
-            if a >= inp_dims or a < -inp_dims:
-                raise ValueError(
-                    f"Not enough dimensions on {input} to reduce on axis {a}"
-                )
-            if a < 0:
-                copy_op = True
-                axis[i] = a + inp_dims
+        copy_op = any(a < 0 for a in axis)
+        # scalar inputs are treated as 1D regarding axis in this `Op`
+        try:
+            axis = np.core.numeric.normalize_axis_tuple(axis, ndim=max(1, inp_dims))
+        except np.AxisError:
+            raise np.AxisError(axis, ndim=inp_dims)
 
         # We can't call self.__class__() as there is a class that
         # inherits from CAReduce that doesn't have the same signature
@@ -1340,13 +1336,12 @@ class CAReduce(COp):
         self.set_ufunc(self.scalar_op)
 
     def __str__(self):
+        prefix = f"{type(self).__name__}{{{self.scalar_op}}}"
         if self.axis is not None:
-            return "Reduce{{{}}}{{{}}}".format(
-                self.scalar_op,
-                ", ".join(str(x) for x in self.axis),
-            )
+            axes_str = ", ".join(str(x) for x in self.axis)
+            return f"{prefix}{{{axes_str}}}"
         else:
-            return "Reduce{%s}" % self.scalar_op
+            return f"{prefix}"
 
     def perform(self, node, inp, out):
         (input,) = inp
@@ -1750,14 +1745,12 @@ class CAReduceDtype(CAReduce):
         return super(CAReduceDtype, op).make_node(input)
 
     def __str__(self):
-        name = self.__class__.__name__
-        if self.__class__.__name__ == "CAReduceDtype":
-            name = ("ReduceDtype{%s}" % self.scalar_op,)
-        axis = ""
+        prefix = f"{type(self).__name__}{{{self.scalar_op}}}"
         if self.axis is not None:
             axis = ", ".join(str(x) for x in self.axis)
-            axis = f"axis=[{axis}], "
-        return f"{name}{{{axis}acc_dtype={self.acc_dtype}}}"
+            return f"{prefix}{{axis=[{axis}], acc_dtype={self.acc_dtype}}}"
+        else:
+            return f"{prefix}{{acc_dtype={self.acc_dtype}}}"
 
 
 def scalar_elemwise(*symbol, nfunc=None, nin=None, nout=None, symbolname=None):
