@@ -13,12 +13,18 @@ import logging
 import warnings
 
 import numpy as np
-from scipy.signal.signaltools import _bvalfromboundary, _valfrommode
-from scipy.signal.sigtools import _convolve2d
+
+
+try:
+    from scipy.signal.signaltools import _bvalfromboundary, _valfrommode
+    from scipy.signal.sigtools import _convolve2d
+except ImportError:
+    from scipy.signal._signaltools import _bvalfromboundary, _valfrommode
+    from scipy.signal._sigtools import _convolve2d
 
 import aesara
 from aesara.graph.basic import Apply
-from aesara.graph.op import OpenMPOp
+from aesara.link.c.op import OpenMPOp
 from aesara.tensor import blas
 from aesara.tensor.basic import (
     as_tensor_variable,
@@ -256,14 +262,6 @@ class ConvOp(OpenMPOp):
         Use a version of c_code that unroll the batch
         (by unroll_batch) and the nkern(by unroll_kern) loop. The size
         must by a multiple of bsize or nkern respectively.
-    verbose : int
-        Passed to GpuConv.
-    version: int or str
-        Passed to GpuConv, if version='no_fft', fft
-        optimization will be deactivated at the op level.
-    direction_hint: {'forward', 'bprop weights', 'bprop inputs'}
-        Passed to GpuConv, used by graph optimizers to aid algorithm choice.
-
     The 3 following parameters are used internally when we generate
     the gradient when dx!=1 or dy!=1.
 
@@ -429,18 +427,9 @@ class ConvOp(OpenMPOp):
         imshp_logical=None,
         kshp_logical=None,
         kshp_logical_top_aligned=True,
-        verbose=0,
-        version=-1,
-        direction_hint="forward",
+        verbose=False,
         openmp=None,
     ):
-        # Deactivate fft_optimization at the op level if specified
-        if version == "no_fft":
-            self.fft_opt = False
-            version = -1
-        else:
-            self.fft_opt = True
-
         # Expand unknown image / kernel shapes into tuples of Nones
         if imshp is None:
             imshp = (None, None, None)
@@ -485,16 +474,13 @@ class ConvOp(OpenMPOp):
         if not all_shape or self.openmp:
             # Only this version is parallelized
             unroll_patch = True
-
+        self.verbose = verbose
         self.imshp = imshp
         self.kshp = kshp
         self.nkern = nkern
         self.bsize = bsize
         self.dx = dx
         self.dy = dy
-        self.verbose = verbose
-        self.version = version
-        self.direction_hint = direction_hint
 
         # a triple
         if imshp_logical is None:
@@ -604,7 +590,7 @@ class ConvOp(OpenMPOp):
 
         self.out_mode = output_mode
 
-        if self.out_mode not in ["valid", "full"]:
+        if self.out_mode not in ("valid", "full"):
             raise NotImplementedError(f"Mode {self.out_mode} not implemented")
 
         if any((shp is not None) and (shp <= 0) for shp in self.outshp):
@@ -681,7 +667,6 @@ class ConvOp(OpenMPOp):
 
     def __setstate__(self, d):
         super().__setstate__(d)
-        self.direction_hint = d.get("direction_hint", None)
         self._rehash()
 
     def _rehash(self):
@@ -758,8 +743,7 @@ class ConvOp(OpenMPOp):
         bcastable23 = [self.outshp[0] == 1, self.outshp[1] == 1]
         output = tensor(
             dtype=_inputs.type.dtype,
-            broadcastable=[_inputs.broadcastable[0], _kerns.broadcastable[0]]
-            + bcastable23,
+            shape=[_inputs.broadcastable[0], _kerns.broadcastable[0]] + bcastable23,
         )
 
         return Apply(self, [_inputs, _kerns], [output])
@@ -772,12 +756,12 @@ class ConvOp(OpenMPOp):
         # replace symbolic shapes with known shapes
         if self.bsize is not None:
             bsize = self.bsize
-        for i in [0, 1, 2]:
+        for i in (0, 1, 2):
             if self.imshp_logical[i] is not None:
                 imshp[i] = self.imshp_logical[i]
         if self.nkern is not None:
             nkern = self.nkern
-        for i in [0, 1]:
+        for i in (0, 1):
             if self.kshp_logical[i] is not None:
                 kshp[i] = self.kshp_logical[i]
         # infer output shape from what we have
@@ -1069,8 +1053,6 @@ class ConvOp(OpenMPOp):
             imshp_logical=imshp_logical,
             kshp_logical=kshp_logical,
             kshp_logical_top_aligned=kshp_logical_top_aligned,
-            version=self.version,
-            direction_hint="bprop weights",
             verbose=self.verbose,
         )
 
@@ -1085,7 +1067,7 @@ class ConvOp(OpenMPOp):
 
         # Determine gradient on inputs ########
         mode = "valid"
-        if not self.out_mode == "full":
+        if self.out_mode != "full":
             mode = "full"
 
         filters = kerns.dimshuffle((1, 0, 2, 3))
@@ -1108,8 +1090,6 @@ class ConvOp(OpenMPOp):
             unroll_patch=None,
             imshp_logical=imshp_logical,
             kshp_logical=None,
-            version=-1,  # we we change the mode, we don't forward the version.
-            direction_hint="bprop inputs",
             verbose=self.verbose,
         )
 
@@ -1416,7 +1396,7 @@ if(%(value)s != %(expected)s){
             d["self_imshp_logical_stride_c"] = int(
                 np.ceil(self.imshp_logical[2] / float(self.imshp[2]))
             )
-            if not self.imshp[0] == 1:
+            if self.imshp[0] != 1:
                 d["affectation"] = "+="
             d["all_shape"] = "1"
             d["dim_zz_const"] = "const"
@@ -1473,7 +1453,7 @@ if(kerns_dim[1] != img2d_dim[1]){
                 f"Type {node.inputs[0].type.dtype} not implemented"
             )
         d["gemm"] = "dgemm_"
-        if not d["type"] == "double":
+        if d["type"] != "double":
             d["gemm"] = "sgemm_"
 
         if self.imshp != self.imshp_logical or self.kshp != self.kshp_logical:
