@@ -6,7 +6,7 @@ import scipy.stats as stats
 
 import aesara
 from aesara.tensor.basic import as_tensor_variable
-from aesara.tensor.random.op import RandomVariable, default_shape_from_params
+from aesara.tensor.random.op import RandomVariable, default_supp_shape_from_params
 from aesara.tensor.random.type import RandomGeneratorType, RandomStateType
 from aesara.tensor.random.utils import broadcast_params
 from aesara.tensor.random.var import (
@@ -25,15 +25,15 @@ except AttributeError:
 
 
 class ScipyRandomVariable(RandomVariable):
-    r"""A class for `RandomVariable`\s that use SciPy-based samplers.
+    r"""A class for straightforward `RandomVariable`\s that use SciPy-based samplers.
 
-    This will only work for `RandomVariable`\s for which the output shape is
-    entirely determined by broadcasting the distribution parameters (e.g. basic
-    scalar distributions).
+    By "straightforward" we mean `RandomVariable`\s for which the output shape
+    is entirely determined by broadcasting the distribution parameters
+    (e.g. basic scalar distributions).
 
     The more sophisticated shape logic performed by `RandomVariable` is avoided
-    in order to reduce the amount of unnecessary extra steps taken to correct
-    SciPy's shape-removing defect.
+    in order to reduce the amount of unnecessary steps taken to correct SciPy's
+    shape-reducing defects.
 
     """
 
@@ -53,12 +53,20 @@ class ScipyRandomVariable(RandomVariable):
     def rng_fn(cls, *args, **kwargs):
         size = args[-1]
         res = cls.rng_fn_scipy(*args, **kwargs)
-        return np.broadcast_to(
-            res,
-            size
-            if size is not None
-            else broadcast_shapes(*[np.shape(a) for a in args[1:-1]]),
-        )
+
+        if np.ndim(res) == 0:
+            # The sample is an `np.number`, and is not writeable, or non-NumPy
+            # type, so we need to clone/create a usable NumPy result
+            return np.asarray(res)
+
+        if size is None:
+            # SciPy will sometimes drop broadcastable dimensions; we need to
+            # check and, if necessary, add them back
+            exp_shape = broadcast_shapes(*[np.shape(a) for a in args[1:-1]])
+            if res.shape != exp_shape:
+                return np.broadcast_to(res, exp_shape).copy()
+
+        return res
 
 
 class UniformRV(RandomVariable):
@@ -559,6 +567,7 @@ class NegBinomialRV(ScipyRandomVariable):
 
 
 nbinom = NegBinomialRV()
+negative_binomial = NegBinomialRV()
 
 
 class BetaBinomialRV(ScipyRandomVariable):
@@ -591,8 +600,8 @@ class MultinomialRV(RandomVariable):
     dtype = "int64"
     _print_name = ("MN", "\\operatorname{MN}")
 
-    def _shape_from_params(self, dist_params, rep_param_idx=1, param_shapes=None):
-        return default_shape_from_params(
+    def _supp_shape_from_params(self, dist_params, rep_param_idx=1, param_shapes=None):
+        return default_supp_shape_from_params(
             self.ndim_supp, dist_params, rep_param_idx, param_shapes
         )
 
@@ -713,7 +722,7 @@ class ChoiceRV(RandomVariable):
     def rng_fn(cls, rng, a, p, replace, size):
         return rng.choice(a, size, replace, p)
 
-    def _shape_from_params(self, *args, **kwargs):
+    def _supp_shape_from_params(self, *args, **kwargs):
         raise NotImplementedError()
 
     def _infer_shape(self, size, dist_params, param_shapes=None):
@@ -803,4 +812,5 @@ __all__ = [
     "triangular",
     "uniform",
     "standard_normal",
+    "negative_binomial",
 ]
