@@ -1,6 +1,7 @@
 from collections.abc import Sequence
 from functools import wraps
 from itertools import zip_longest
+from typing import Optional, Union
 
 import numpy as np
 
@@ -111,7 +112,9 @@ def broadcast_params(params, ndims_params):
     return bcast_params
 
 
-def normalize_size_param(size):
+def normalize_size_param(
+    size: Optional[Union[int, np.ndarray, Variable, Sequence]]
+) -> Variable:
     """Create an Aesara value for a ``RandomVariable`` ``size`` parameter."""
     if size is None:
         size = constant([], dtype="int64")
@@ -130,6 +133,7 @@ def normalize_size_param(size):
             # `Scan` performs)
             size = specify_shape(size, (get_vector_length(size),))
 
+    assert not any(s is None for s in size.type.shape)
     assert size.dtype in int_dtypes
 
     return size
@@ -217,7 +221,7 @@ class RandomStream:
         self.gen_seedgen = np.random.default_rng(seed)
 
         for old_r, new_r in self.state_updates:
-            old_r_seed = self.gen_seedgen.integers(2 ** 30)
+            old_r_seed = self.gen_seedgen.integers(2**30)
             old_r.set_value(self.rng_ctor(int(old_r_seed)), borrow=True)
 
     def gen(self, op, *args, **kwargs):
@@ -246,7 +250,7 @@ class RandomStream:
             )
 
         # Generate a new random state
-        seed = int(self.gen_seedgen.integers(2 ** 30))
+        seed = int(self.gen_seedgen.integers(2**30))
         random_state_variable = shared(self.rng_ctor(seed))
 
         # Distinguish it from other shared variables (why?)
