@@ -32,7 +32,7 @@ from aesara.tensor.basic_opt import (
 from aesara.tensor.elemwise import Elemwise
 from aesara.tensor.exceptions import NotScalarConstantError
 from aesara.tensor.math import Dot, add
-from aesara.tensor.math import all as aet_all
+from aesara.tensor.math import all as at_all
 from aesara.tensor.math import (
     and_,
     ceil_intdiv,
@@ -71,6 +71,7 @@ from aesara.tensor.subtensor import (
     get_idx_list,
     get_slice_elements,
     inc_subtensor,
+    indices_from_subtensor,
 )
 from aesara.tensor.type import TensorType
 from aesara.tensor.type_other import NoneTypeT, SliceConstant, SliceType
@@ -88,7 +89,7 @@ def register_useless(lopt, *tags, **kwargs):
         name = kwargs.pop("name", None) or lopt.__name__
 
         compile.mode.local_useless.register(
-            name, lopt, "last", "fast_run", *tags, **kwargs
+            name, lopt, "fast_run", *tags, position="last", **kwargs
         )
         return lopt
 
@@ -224,9 +225,6 @@ def local_replace_AdvancedSubtensor(fgraph, node):
         return
 
     new_res = transform_take(indexed_var, indices[axis], axis)
-
-    assert new_res.broadcastable == node.outputs[0].broadcastable
-
     copy_stack_trace(node.outputs[0], new_res)
     return [new_res]
 
@@ -271,7 +269,7 @@ def local_AdvancedIncSubtensor_to_AdvancedIncSubtensor1(fgraph, node):
 @register_specialize
 @local_optimizer([Subtensor])
 def local_subtensor_of_dot(fgraph, node):
-    """Rewrite ``aet.dot(A, B)[idxs]`` into ``aet.dot(A[idxs_a], B[idxs_b])``.
+    """Rewrite ``at.dot(A, B)[idxs]`` into ``at.dot(A[idxs_a], B[idxs_b])``.
     ``idxs_a`` is the first ``A.ndim-1`` entries of ``idxs``, and ``idxs_b`` is
     the remaining entries of ``idxs`` (if any), modified to skip the
     second-to-last dimension of ``B`` (because dot sums over this dimension).
@@ -392,7 +390,7 @@ def local_subtensor_lift(fgraph, node):
 
         if isinstance(u.owner.op, Elemwise):
             new_inputs = []
-            if all([sum(i.type.broadcastable) == 0 for i in u.owner.inputs]):
+            if all(sum(i.type.broadcastable) == 0 for i in u.owner.inputs):
                 # There is no broadcastable in the inputs
                 idx = node.inputs[1:]
                 new_inputs = [node.op(i, *idx) for i in u.owner.inputs]
@@ -404,9 +402,7 @@ def local_subtensor_lift(fgraph, node):
                 # and stacktrace from previous unary operation
                 copy_stack_trace([node.outputs[0], node.inputs[0]], ret)
                 return [ret]
-            elif all(
-                [sum(i.type.broadcastable) in [i.ndim, 0] for i in u.owner.inputs]
-            ):
+            elif all(sum(i.type.broadcastable) in [i.ndim, 0] for i in u.owner.inputs):
                 # There is no broadcastable in the inputs or it is scalar
                 idx = node.inputs[1:]
                 new_inputs = []
@@ -538,7 +534,7 @@ def local_subtensor_merge(fgraph, node):
 
             # Restore original broadcastable dimensions that `subtens()` may
             # have been unable to infer again
-            if out.type != orig_out.type:
+            if not orig_out.type.is_super(out.type):
                 assert out.dtype == orig_out.dtype
                 assert out.ndim == orig_out.ndim
                 out = patternbroadcast(out, orig_out.broadcastable)
@@ -566,13 +562,13 @@ def local_subtensor_remove_broadcastable_index(fgraph, node):
     remove_dim = []
     node_inputs_idx = 1
     for dim, elem in enumerate(idx):
-        if isinstance(elem, (aes.Scalar)):
-            # The idx is a Scalar, ie a Type. This means the actual index
+        if isinstance(elem, (aes.ScalarType)):
+            # The idx is a ScalarType, ie a Type. This means the actual index
             # is contained in node.inputs[1]
             dim_index = node.inputs[node_inputs_idx]
             if isinstance(dim_index, aes.ScalarConstant):
                 dim_index = dim_index.value
-            if dim_index in [0, -1] and node.inputs[0].broadcastable[dim]:
+            if dim_index in (0, -1) and node.inputs[0].broadcastable[dim]:
                 remove_dim.append(dim)
                 node_inputs_idx += 1
             else:
@@ -581,7 +577,7 @@ def local_subtensor_remove_broadcastable_index(fgraph, node):
             if elem != slice(None):
                 return
         elif isinstance(elem, (int, np.integer)):
-            if elem in [0, -1] and node.inputs[0].broadcastable[dim]:
+            if elem in (0, -1) and node.inputs[0].broadcastable[dim]:
                 remove_dim.append(dim)
         else:
             raise TypeError("case not expected")
@@ -658,9 +654,9 @@ def local_subtensor_of_alloc(fgraph, node):
     if nw_val.ndim > len(nw_dims):
         return False
     rval = alloc(nw_val, *nw_dims)
-    if type(rval) not in (list, tuple):
+    if not isinstance(rval, (list, tuple)):
         rval = [rval]
-    if rval[0].type != node.outputs[0].type:
+    if not node.outputs[0].type.is_super(rval[0].type):
         # It happen that the make_node() isn't able to infer the same pattern.
         # We know it is safe, so fix that.
         rval[0] = patternbroadcast(rval[0], node.outputs[0].broadcastable)
@@ -691,7 +687,10 @@ def local_subtensor_inc_subtensor(fgraph, node):
             # If the dtypes differ, cast y into x.dtype
             if x.dtype != y.dtype:
                 y = y.astype(x.dtype)
-            if out.type == y.type:
+            if (
+                out.type.dtype == y.type.dtype
+                and out.type.broadcastable == y.type.broadcastable
+            ):
                 # if x[idx] and y have the same type, directly return y
                 return [y]
             else:
@@ -705,7 +704,7 @@ def local_subtensor_inc_subtensor(fgraph, node):
 
 
 @register_specialize
-@register_canonicalize("fast_compile_gpu")
+@register_canonicalize("fast_compile")
 @register_useless
 @local_optimizer([Subtensor, AdvancedSubtensor1])
 def local_subtensor_make_vector(fgraph, node):
@@ -742,9 +741,9 @@ def local_subtensor_make_vector(fgraph, node):
     if isinstance(node.op, Subtensor):
         (idx,) = node.op.idx_list
 
-        if isinstance(idx, (aes.Scalar, TensorType)):
+        if isinstance(idx, (aes.ScalarType, TensorType)):
             old_idx, idx = idx, node.inputs[1]
-            assert idx.type == old_idx
+            assert idx.type.is_super(old_idx)
     elif isinstance(node.op, AdvancedSubtensor1):
         idx = node.inputs[1]
 
@@ -787,34 +786,38 @@ def local_subtensor_make_vector(fgraph, node):
 @register_specialize
 @local_optimizer([IncSubtensor])
 def local_useless_inc_subtensor(fgraph, node):
-    """
-    Remove IncSubtensor, when we overwrite the full inputs with the
-    new value.
+    r"""Remove redundant `IncSubtensor`\s.
 
+    More specifically, ``set_subtensor(x[indices], y)`` is replaced by
+    ``y[indices]`` when ``indices`` are full `slice`\s and ``y``'s shape is
+    equal to ``x[indices]``, and ``inc_subtensor(x[indices], y)`` is replaced
+    by ``y[indices]`` when ``x[indices]`` is some array of ``0``\s, ``indices``
+    are full slices, and the shapes are equal.
     """
     if not isinstance(node.op, IncSubtensor):
         return
+
+    if not hasattr(fgraph, "shape_feature"):
+        return
+
+    x, y, *index_inputs = node.inputs
+
     if node.op.set_instead_of_inc is False:
-        # This is an IncSubtensor, so the init value must be zeros
+        # This is an increment operation, so the array being incremented must
+        # consist of all zeros in order for the entire operation to be useless
         try:
-            c = get_scalar_constant_value(node.inputs[0], only_process_constants=True)
+            c = get_scalar_constant_value(x)
             if c != 0:
                 return
         except NotScalarConstantError:
             return
-    if (
-        node.inputs[0].ndim != node.inputs[1].ndim
-        or node.inputs[0].broadcastable != node.inputs[1].broadcastable
-    ):
-        # FB: I didn't check if this case can happen, but this opt
-        # don't support it.
-        return
-    # We have a SetSubtensor or an IncSubtensor on zeros
-    # If is this IncSubtensor useful?
 
-    # Check that we keep all the original data.
-    # Put the constant inputs in the slice.
-    idx_cst = get_idx_list(node.inputs[1:], node.op.idx_list)
+    idx_cst = indices_from_subtensor(list(index_inputs), node.op.idx_list)
+
+    # Check that all indices are full slices with only reversals and no step
+    # sizes
+    # TODO: It seems like there should be a basic `IncSubtensor`
+    # canonicalization that removes these redundant slices.
     if all(
         isinstance(e, slice)
         and e.start is None
@@ -825,20 +828,22 @@ def local_useless_inc_subtensor(fgraph, node):
         )
         for e in idx_cst
     ):
-        # IncSubtensor broadcast node.inputs[1] on node.inputs[0]
-        # based on run time shapes, so we must check they are the same.
-        if not hasattr(fgraph, "shape_feature"):
+
+        # `IncSubtensor` broadcasts `x` on `y` based on run-time shapes, so we
+        # must check that they are the same
+        if not fgraph.shape_feature.same_shape(x, y):
             return
-        if not fgraph.shape_feature.same_shape(node.inputs[0], node.inputs[1]):
-            return
-        # There is no reverse, so we don't need a replacement.
+
+        # There are no reversals, so we don't need a replacement.
         if all(e.step is None for e in node.op.idx_list):
-            # They are the same shape, so we can remove this IncSubtensor
-            return [node.inputs[1]]
-        ret = Subtensor(node.op.idx_list)(*node.inputs[1:])
-        # Copy over previous output stacktrace
-        copy_stack_trace(node.outputs, ret)
-        return [ret]
+            # They are exactly the same shapes, so we can remove this `IncSubtensor`
+            return [y]
+
+        new_node = Subtensor(node.op.idx_list).make_node(y, *index_inputs)
+        new_out = new_node.outputs[0]
+        copy_stack_trace(node.outputs, new_out)
+
+        return [new_out]
 
 
 @register_canonicalize
@@ -1180,7 +1185,7 @@ def local_IncSubtensor_serialize(fgraph, node):
                     AdvancedIncSubtensor,
                 ),
             )
-            and i.type == o_type
+            and i.type.is_super(o_type)
             and len(fgraph.clients[i]) == 1
             and not i.owner.op.set_instead_of_inc
         )
@@ -1207,8 +1212,8 @@ def local_IncSubtensor_serialize(fgraph, node):
             # stack up the new incsubtensors
             tip = new_add
             for mi in movable_inputs:
-                assert tip.type == o_type
-                assert tip.type == mi.owner.inputs[0].type
+                assert o_type.is_super(tip.type)
+                assert mi.owner.inputs[0].type.is_super(tip.type)
                 tip = mi.owner.op(tip, *mi.owner.inputs[1:])
                 # Copy over stacktrace from outputs of the original
                 # "movable" operation to the new operation.
@@ -1225,9 +1230,9 @@ def local_IncSubtensor_serialize(fgraph, node):
 compile.optdb.register(
     "pre_local_IncSubtensor_serialize",
     in2out(local_IncSubtensor_serialize),
-    # Just before canonizer
-    0.99,
     "fast_run",
+    # Just before canonizer
+    position=0.99,
 )
 
 
@@ -1262,9 +1267,9 @@ compile.optdb.register(
     TopoOptimizer(
         local_inplace_setsubtensor, failure_callback=TopoOptimizer.warn_inplace
     ),
-    60,
     "fast_run",
     "inplace",
+    position=60,
 )
 
 
@@ -1283,9 +1288,9 @@ compile.optdb.register(
     TopoOptimizer(
         local_inplace_AdvancedIncSubtensor1, failure_callback=TopoOptimizer.warn_inplace
     ),
-    60,
     "fast_run",
     "inplace",
+    position=60,
 )
 
 
@@ -1308,9 +1313,9 @@ compile.optdb.register(
     TopoOptimizer(
         local_inplace_AdvancedIncSubtensor, failure_callback=TopoOptimizer.warn_inplace
     ),
-    60,
     "fast_run",
     "inplace",
+    position=60,
 )
 
 
@@ -1440,7 +1445,7 @@ def local_adv_sub1_adv_inc_sub1(fgraph, node):
     if not inp.owner.op.set_instead_of_inc:
         return
 
-    cond = [aet_all(and_(lt(idx, x.shape[0]), ge(idx, -x.shape[0])))]
+    cond = [at_all(and_(lt(idx, x.shape[0]), ge(idx, -x.shape[0])))]
     if not fgraph.shape_feature.same_shape(idx, y, 0, 0):
         cond.append(eq(idx.shape[0], y.shape[0]))
     r = Assert(
@@ -1479,7 +1484,7 @@ def local_useless_inc_subtensor_alloc(fgraph, node):
         i = node.inputs[2:]
 
         if y.owner is not None and isinstance(y.owner.op, Alloc):
-            # `z` is the input of the Alloc op, i.e. aet.alloc(z, <shape>)
+            # `z` is the input of the Alloc op, i.e. at.alloc(z, <shape>)
             z = y.owner.inputs[0]
 
             try:
@@ -1627,7 +1632,7 @@ def local_subtensor_shape_constant(fgraph, node):
 @register_canonicalize
 @local_optimizer([Subtensor])
 def local_subtensor_SpecifyShape_lift(fgraph, node):
-    """Lift ``specify_shape(x, s)[i]`` to ``specify_shape(x[i], s[i])``."""
+    """Lift ``specify_shape(x, s)[i_1, ..., i_n]`` to ``specify_shape(x[i1, ... , i_n], s[n:])``."""
 
     if not isinstance(node.op, Subtensor):
         return False
@@ -1641,23 +1646,18 @@ def local_subtensor_SpecifyShape_lift(fgraph, node):
         return False
 
     obj_arg = specify_shape_node.owner.inputs[0]
-    shape_arg = specify_shape_node.owner.inputs[1]
+    shape_arg = specify_shape_node.owner.inputs[1:]
 
     indices = get_idx_list(node.inputs, node.op.idx_list)
 
-    if len(indices) > 1 or len(indices) == 0:
-        return False
-
-    if isinstance(indices[0], slice) or isinstance(
-        getattr(indices[0], "type", None), SliceType
+    if any(
+        isinstance(index, slice) or isinstance(getattr(index, "type", None), SliceType)
+        for index in indices
     ):
         return False
 
     new_obj_arg = obj_arg[indices]
-
+    # No need to specify shape for scalar outputs
     if new_obj_arg.ndim == 0:
-        new_shape_arg = as_tensor([], dtype=np.int64)
-    else:
-        new_shape_arg = as_tensor(shape_arg[indices], dtype=np.int64, ndim=1)
-
-    return [specify_shape(new_obj_arg, new_shape_arg)]
+        return [new_obj_arg]
+    return [specify_shape(new_obj_arg, shape_arg[len(indices) :])]
